package plugin;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.AttributeHelper;
import org.BackgroundTaskStatusProviderSupportingExternalCall;
import org.apache.commons.collections15.Transformer;
import org.graffiti.attributes.CollectionAttribute;
import org.graffiti.editor.MainFrame;
import org.graffiti.event.GraphEvent;
import org.graffiti.event.GraphListener;
import org.graffiti.event.TransactionEvent;
import org.graffiti.graph.AdjListGraph;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;
import org.graffiti.plugin.inspector.InspectorTab;
import org.graffiti.plugin.view.View;
import org.graffiti.selection.SelectionEvent;
import org.graffiti.selection.SelectionListener;
import org.graffiti.selection.SelectionModel;
import org.graffiti.session.EditorSession;
import org.graffiti.session.Session;
import org.graffiti.session.SessionListener;

import de.ipk_gatersleben.ag_nw.centilib.plugin.CentiLibInteractions;
import de.ipk_gatersleben.ag_nw.centilib.plugin.CentiLibPlugin;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DoubleAttribute;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.misc.invert_selection.AttributePathNameSearchType;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.misc.invert_selection.SearchAndSelecAlgorithm;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.misc.invert_selection.SearchAttributeHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.misc.invert_selection.SearchType;

/**
 * This class extends the InspectorTab needed to integrate a new panel into Vanted and implements various listeners 
 * and all interfaces needed to use the CentiLib GUI (except the graph wrapper). As a consequence CentiLibTab joines the plugin interfaces of
 * Vanted and CentiLib.
 *  
 * @author Johannes Graessler
 *
 */

@SuppressWarnings("serial")
public class CentiLibTab extends InspectorTab implements
	GraphListener,
	SelectionListener,
	SessionListener,
	CentiLibInteractions<Graph,Node,Edge>{

	public final String attributePath = "centralities";
	
	private CentiLibPlugin<Graph,Node, Edge> centilib;
	
	public CentiLibTab(){
		super();
		
		// implementing 'Transformer' inline 
		Transformer<Node, String> nodeLabel =
			new Transformer<Node,String>(){
					public String transform(Node node){
						return (AttributeHelper.getLabel(node, ""));
					}
				};
				
		Transformer<Graph, String> graphName = 
			new Transformer<Graph, String>(){
				public String transform(Graph graph){
					return graph.getName();
				}
			};
		
		// generating the CentiLib GUI or the plugin respectively.
		this.centilib = 
			new CentiLibPlugin<Graph,Node,Edge>(
					this,		//CentiLibInteractions
					graphName,	//Transformer<Graph, String>
					nodeLabel	//Transformer<Node, String>
				);		
		
		this.setLayout(new BorderLayout());
		this.title = "CentiLib";
		
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = GridBagConstraints.RELATIVE;
        gbc.gridx = 0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 0.0;
        gbc.weighty = 0.0;

        // adding the CentiLib panel as the only graphical component in this panel
        this.add(this.centilib.getPanel());
        
        validate();
		return;
	}
	
	@Override
	public boolean visibleForView(View v) {
		// only show the centilib panel if there is at least one graph view
		if(v == null)
			return false;
		return true;
	}
	
	//-> CentiLibInteractions 
	public void selectGraph(Graph graph){
		
		for(Session s : MainFrame.getSessions())
			if(s.getGraph() == graph)
				MainFrame.getInstance().setActiveSession(s, s.getActiveView());

	}

	@Override //->CentiLibInteractions
	public void selectNodesOfCurrentGraph(Set<Node> vertices) {
	
		if(MainFrame.getInstance().getActiveEditorSession() != null){
			SelectionModel sm = MainFrame.getInstance().getActiveEditorSession().getSelectionModel();
			if(sm != null){
				sm.getActiveSelection().clear();
				sm.getActiveSelection().addAll(vertices);
				sm.selectionChanged();
			}
		}
	}
	
	@Override//-> CentiLibInteractions
	public boolean addCentralityAttribute(Node node,  String centrality, Double value){
		
		String attrName = this.makeAttributeName(centrality);
		AttributeHelper.setAttribute(node, attributePath, attrName, value);		
		node.getGraph().setModified(true);
		MainFrame.getInstance().getActiveEditorSession().getSelectionModel().selectionChanged();
		return true;
	}
	
	@Override //-> CentiLibInteractions
	public void deleteCentralityAttribute(Node node, String cent){
		
		//delete the given centrality attribute
		String attrName = makeAttributeName(cent);
		if(AttributeHelper.hasAttribute(node, attributePath,attrName))
			AttributeHelper.deleteAttribute(node, attributePath, attrName);
		
		//if there are no more centrality attributes left
		// -> delete the centrality attribute path
		if(AttributeHelper.hasAttribute(node, attributePath))
			if(((CollectionAttribute)node.getAttribute(attributePath)).getCollection().size() == 0)
				AttributeHelper.deleteAttribute(node, "", attributePath+"*");
		
	}

	@Override//->CentiLibInteractions
	public Graph createSubGraph(Graph graph, Set<Node> nodes, Set<Edge> edges, String name){

		if(nodes == null || edges == null) return null;
		
		Graph newGraph = new AdjListGraph();
		
		Node source,target;
		
		// map the original nodes to the new ones to be able to create correct edges
		Map<Node,Node> added = new HashMap<Node,Node>();
		
		
		for(Edge e: edges){
			
			//if source or target node was not added to the new graph, do it now
			if( added.get(e.getSource()) == null){
				source = newGraph.addNodeCopy(e.getSource());
				added.put(e.getSource(), source);
			}
			
			if( added.get(e.getTarget()) == null){
				target = newGraph.addNodeCopy(e.getTarget());
				added.put(e.getTarget(),target);
			}
			
			// add an edge copy between the nodes that were created for the source and target of e 
			newGraph.addEdgeCopy(e, added.get(e.getSource()), added.get(e.getTarget()));
			
		}
		
		//if there are nodes without any edges
		if(nodes.size() != added.size())
			for(Node n : nodes)
				if(!added.containsKey(n))
					added.put(n, newGraph.addNodeCopy(n));
		
		newGraph.setName(name);

//		String dv = MainFrame.getInstance().getDefaultView();
	//	if(dv != null)
      //      MainFrame.getInstance().createInternalFrame(dv, "", false, false);
        //else
            MainFrame.getInstance().showViewChooserDialog(new EditorSession(newGraph), false, null);
		
		return newGraph;
	}
	
	@Override //->CentiLibInteractions
	public Set<DoubleAttribute<Node> > getNumericNodeAttributes(Graph graph){
		
		if(graph == null)
			return null;
		
		SearchAttributeHelper sah = new SearchAttributeHelper();
		ArrayList<AttributePathNameSearchType> possibleAttributes = new ArrayList<AttributePathNameSearchType>();
		try {
			sah.prepareSearch();
			SearchAndSelecAlgorithm.enumerateAttributes(possibleAttributes, graph.getNodes(), SearchType.getSetOfNumericSearchTypes());

		} finally {
			sah.restoreDefintions();
		}				
		
		Set<DoubleAttribute<Node> > attributes = new HashSet<DoubleAttribute<Node> >();
		
		for(AttributePathNameSearchType attribute : possibleAttributes)
			attributes.add(new VantedNodeAttribute(attribute));
		
		return attributes;
		
	}
	
	@Override //->CentiLibInteractions
	public Set<DoubleAttribute<Edge> > getNumericEdgeAttributes(Graph graph){
		
		if(graph == null)
			return null;
		
		SearchAttributeHelper sah = new SearchAttributeHelper();
		ArrayList<AttributePathNameSearchType> possibleAttributes = new ArrayList<AttributePathNameSearchType>();
		try {
			sah.prepareSearch();
			SearchAndSelecAlgorithm.enumerateAttributes(possibleAttributes, graph.getEdges(), SearchType.getSetOfNumericSearchTypes());

		} finally {
			sah.restoreDefintions();
		}				
		
		Set<DoubleAttribute<Edge> > attributes = new HashSet<DoubleAttribute<Edge> >();
		
		for(AttributePathNameSearchType attribute : possibleAttributes)
			attributes.add(new VantedEdgeAttribute(attribute));
		
		return attributes;
	}
	
	@Override //-> CentiLibInteractions
	public Graph getCurrentGraph(){
		
		if(MainFrame.getInstance().getActiveEditorSession() == null)
			return null;
		return MainFrame.getInstance().getActiveEditorSession().getGraph();
	}
	
	@Override //->CentiLibInteractions
	public Set<Graph> getGraphs(){
		Set<Graph> graphs = new HashSet<Graph>();
		
		for(Session t : MainFrame.getEditorSessions())
			graphs.add(t.getGraph());
		return graphs;
	}
	
	@Override //->CentiLibInteractions
	public edu.uci.ics.jung.graph.Graph<Node, Edge> getWrapper(Graph graph){
		if(graph != null)
			return new VantedNetworkWrapper<Node, Edge>(graph);
		
		return null;
	}
	
	/**
	 * This method removes some special characters from a string which are not allowed within Vanted attribute names.
	 *  
	 * @param name original attribute name
	 * @return attribute name that is compatible with Vanted
	 */
	protected String makeAttributeName(String name) {
		
		String newName = "";
		
		//delete blanks and hyphens
		for(int i =0; i<name.length();i++)
			if( name.charAt(i) != ' ' &&
				name.charAt(i) != '-'
			)
				newName += name.charAt(i);
		
		newName = newName.replace('.', 'p');
		newName = newName.replace(',', 'p');
		
		return newName;
		
	}	

	
	protected class VantedEdgeAttribute implements DoubleAttribute<Edge>{
		
		private AttributePathNameSearchType attribute;
		
		public VantedEdgeAttribute(AttributePathNameSearchType attr){
			this.attribute = attr;
		}
		
		@Override
		public String toString(){
			return attribute.toString();
		}
		
		public String getName(){
			return attribute.getAttributeName();
		}
		@Override
		public Double getValue(Edge edge){
			Double result = attribute.getAttributeValue(edge, Double.NaN);
			if(result.isNaN())
				return null;
			return result;
		}

	};

	protected class VantedNodeAttribute implements DoubleAttribute<Node>{
		
		private AttributePathNameSearchType attribute;
		private String nameString = "";
		private String centName = "";
		
		public VantedNodeAttribute(AttributePathNameSearchType attr){
			this.attribute = attr;
			

		}
		
		@Override
		public String toString(){
			if (!nameString.equals(""))
				return nameString;
			
			nameString = getName();
		
			nameString = attribute.toString().replace(attribute.getAttributeName(), nameString);
			
			return nameString;
		}
		
		public String getName(){
			if(!centName.equals(""))
				return centName;
			
			centName = attribute.getAttributeName();
			
			if(centName.length() < 3)
				return centName;
			
			//if there is a parameter value with one post decimal places, we will replace the 'p' with a dot
			if(centName.charAt(centName.length()-2) == 'p')
				centName = centName.substring(0,centName.length()-2)+"."+centName.substring(centName.length()-1);
			
			//if there is a parameter value with two post decimal places, we will replace the 'p' with a dot
			if(centName.charAt(centName.length()-3) == 'p')
				centName = centName.substring(0,centName.length()-3)+"."+centName.substring(centName.length()-2);
			
			return centName;
			
		}
		
		@Override
		public Double getValue(Node node){
			Double result = attribute.getAttributeValue(node, Double.NaN);
			if(result.isNaN())
				return null;
			return result;
		}

	};
	
	
	// ***************	Listeners	*****************
	
	@Override
	public void transactionFinished(TransactionEvent e,
			BackgroundTaskStatusProviderSupportingExternalCall status) {
		
		String temp = e.getSource().getClass().getSimpleName();
		if(temp.equals("GraphElementsDeletionEdit"))
			centilib.postGraphChanged(getCurrentGraph());
	}
		
	@Override
	public void transactionStarted(TransactionEvent e) {}
		


	@Override
	public void sessionDataChanged(Session s) {
		sessionChanged(s);
	}
	
	@Override
	public void sessionChanged(Session s) {
		
		if (s==null){
			setGraph(null);
		}
		else
			setGraph(s.getGraph());
	}
	
	private void setGraph(final Graph graph){
		centilib.graphFocused(graph);
		if(graph != null)
			graph.getListenerManager().addDelayedGraphListener(this);
	}
	
	
	
	@Override
	public void selectionChanged(SelectionEvent e) {
	
		centilib.nodesSelected(
			MainFrame.getInstance().getActiveEditorSession().getSelectionModel().getActiveSelection().getNodes());
	}
	
	@Override
	public boolean isSelectionListener(){ 
		return true;
	}

	@Override
	public void preEdgeAdded(GraphEvent e) {
		centilib.preGraphChanged(getCurrentGraph());
	}
	
	@Override
	public void preEdgeRemoved(GraphEvent e) {
		centilib.preGraphChanged(getCurrentGraph());
	}

	@Override
	public void preGraphCleared(GraphEvent e) {
		centilib.preGraphChanged(getCurrentGraph());
	}

	@Override
	public void preNodeAdded(GraphEvent e) {
		centilib.preGraphChanged(getCurrentGraph());
	}

	@Override
	public void preNodeRemoved(GraphEvent e) {
		centilib.preGraphChanged(getCurrentGraph());
		
	}

	@Override
	public void selectionListChanged(SelectionEvent e) {}

	@Override
	public void postEdgeAdded(GraphEvent e) {}

	@Override
	public void postEdgeRemoved(GraphEvent e) {}

	@Override
	public void postGraphCleared(GraphEvent e) {}

	@Override
	public void postNodeAdded(GraphEvent e) {}

	@Override
	public void postNodeRemoved(GraphEvent e) {}

}
