package plugin;

import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.graph.util.Pair;
import giny.model.Edge;
import giny.model.Node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.Semantics;


public class CyNetworkWrapper<V extends Node,E extends Edge> implements Graph<Node,Edge> {

	protected CyNetwork graph;
	
	public CyNetworkWrapper(CyNetwork graph) {
		this.graph = graph;
	}
	
	public void setNetwork(CyNetwork graph){
		this.graph = graph;
	}

	public CyNetwork getNetwork(){
		return graph;
	}
	@Override
	public boolean addEdge(Edge edge, Node source, Node target) {

		if(edge.isDirected())
			return addEdge(edge,source,target,EdgeType.DIRECTED);
		return addEdge(edge,source,target,EdgeType.DIRECTED);
	}

	@Override
	public boolean addEdge(Edge edge, Node source, Node target, EdgeType et) {

		if(!graph.containsNode(source) || !graph.containsNode(target))
			return false;
		if(graph.containsEdge(edge))
			return true;
		if(et == EdgeType.DIRECTED)
			graph.addEdge(Cytoscape.getCyEdge(source,target,Semantics.INTERACTION,"DirectedEdge",true,true));
		else
			graph.addEdge(Cytoscape.getCyEdge(source,target,Semantics.INTERACTION,"UndirectedEdge",true,false));
		return true;
	}

	@Override
	public Node getDest(Edge edge) {
		
		if(edge.isDirected())
			return edge.getTarget();
		return null;
	}

	@Override
	public Pair<Node> getEndpoints(Edge edge) {
		if(!graph.containsEdge(edge))
			return null;
		return new Pair<Node>(edge.getSource(),edge.getTarget());
	}

	@Override
	public Collection<Edge> getInEdges(Node node) {

		int[] inEdges= graph.getAdjacentEdgeIndicesArray(graph.getIndex(node), true, true, false);
		return intArrayToEdgeCollection(inEdges);
	}

	@Override
	public Node getOpposite(Node node, Edge edge) {

		if(edge.getTarget().equals(node))
			return edge.getSource();
		else return edge.getTarget();
	}

	/*
	 * 
	 * Get all undirected edges and all directed edges starting from this node.
	 * @see edu.uci.ics.jung.graph.Graph#getOutEdges(java.lang.Object)
	 */
	@Override
	public Collection<Edge> getOutEdges(Node node) {
		
		int[] inEdges = graph.getAdjacentEdgeIndicesArray(graph.getIndex(node), true, false, true);
		return intArrayToEdgeCollection(inEdges);
	}

	@Override
	public int getPredecessorCount(Node node) {
		return getPredecessors(node).size();
	}

	@Override
	public Collection<Node> getPredecessors(Node node) {

		Collection<Edge> inEdges = getInEdges(node);
		Collection<Node> predecessors = new ArrayList<Node>(inEdges.size());
		for(Edge edge : inEdges){
			if(!predecessors.contains(getOpposite(node, edge)))
				predecessors.add(getOpposite(node, edge));
		}
		return predecessors;
	}

	@Override
	public Node getSource(Edge edge) {
		if(edge.isDirected())
			return edge.getSource();
		return null;
	}

	@Override
	public int getSuccessorCount(Node node) {
		return getSuccessors(node).size();
	}

	@Override
	public Collection<Node> getSuccessors(Node node) {
		Collection<Edge> outEdges = getOutEdges(node);
		Collection<Node> successors = new ArrayList<Node>(outEdges.size());
		for(Edge edge : outEdges){
			if(!successors.contains(getOpposite(node, edge)))
			successors.add(getOpposite(node, edge));
		}
		return successors;
	}

	@Override
	public int inDegree(Node node) {
		return graph.getInDegree(node,true);
	}

	@Override
	public boolean isDest(Node node, Edge edge) {
		if(getDest(edge) == null)
			return false;
		return getDest(edge).equals(node);
	}

	@Override
	public boolean isPredecessor(Node target, Node source) {
		return getPredecessors(target).contains(source);
	}

	@Override
	public boolean isSource(Node node, Edge edge) {
		if(getSource(edge) == null)
			return false;
		return getSource(edge).equals(node);
	}

	@Override
	public boolean isSuccessor(Node target, Node source) {
		return getSuccessors(source).contains(target);
	}

	@Override
	public int outDegree(Node node) {
		return graph.getOutDegree(node,true);
	}

	@Override
	public boolean addEdge(Edge edge, Collection<? extends Node> nodes) {

		if(edge.isDirected())
			return addEdge(edge,nodes,EdgeType.DIRECTED);
		return addEdge(edge,nodes,EdgeType.UNDIRECTED);
	}

	@Override
	public boolean addEdge(Edge edge, Collection<? extends Node> nodes,
			EdgeType et) {
		
		if(nodes.size() != 2)
			return false;
		Iterator<? extends Node> it = nodes.iterator();
		return addEdge(edge,it.next(),it.next(),et);
	}

	@Override
	public boolean addVertex(Node node) {
		if(graph.addNode(node) != null)
			return true;
		return false;
	}

	@Override
	public boolean containsEdge(Edge edge) {
		return graph.containsEdge(edge);
	}

	@Override
	public boolean containsVertex(Node node) {
		return graph.containsNode(node);
	}

	@Override
	public int degree(Node node) {
		return graph.getDegree(node);
	}

	@Override
	public Edge findEdge(Node source, Node target) {

		Edge temp = Cytoscape.getCyEdge(source, target, Semantics.INTERACTION,"pp",false,true);
		if(graph.containsEdge(temp) && graph.containsNode(source) && graph.containsNode(target))
			return temp;
		
		return null;
	}

	@Override
	public Collection<Edge> findEdgeSet(Node source, Node target) {
		
		List<Edge> list = (List<Edge>) graph.edgesList(source, target);
		if(list == null)
			return null;
		return listToEdgeCollection(list);

	}

	@Override
	public int getEdgeCount() {
		return graph.getEdgeCount();
	}

	@Override
	public int getEdgeCount(EdgeType et) {
		
		boolean directed = false;
		int counter = 0;
		if(et == EdgeType.DIRECTED)
			directed = true;
		
		for(Iterator<Edge> it = graph.edgesIterator();it.hasNext(); ){
			if(it.next().isDirected() == directed)
				counter++;
		}
		return counter;
	}

	@Override
	public EdgeType getEdgeType(Edge edge) {
		if(edge.isDirected())
			return EdgeType.DIRECTED;
		else return EdgeType.UNDIRECTED;
	}

	@Override
	public Collection<Edge> getEdges() {
		
		List<Edge> list = graph.edgesList();
		return listToEdgeCollection(list);
	}

	@Override
	public Collection<Edge> getEdges(EdgeType et) {

		boolean directed = false;
		Collection<Edge> edges = new ArrayList<Edge>();
		
		if(et == EdgeType.DIRECTED)
			directed = true;

		Edge edge;
		for(Iterator<Edge> it = graph.edgesIterator();it.hasNext(); ){
			edge = (Edge)it.next();
			if(edge.isDirected() == directed)
				edges.add(edge);
		}
		return edges;
	}

	@Override
	public int getIncidentCount(Edge edge) {
		if(edge.getSource() == edge.getTarget())
			return 1;
		return 2;
	}

	/*
	 * Returns directed and undirected edges, which end in node
	 * @see edu.uci.ics.jung.graph.Hypergraph#getIncidentEdges(java.lang.Object)
	 */
	@Override
	public Collection<Edge> getIncidentEdges(Node node) {
		int[] edgesList = graph.getAdjacentEdgeIndicesArray(graph.getIndex(node), true, true, false);
		if(edgesList == null)
			return null;
		
		return intArrayToEdgeCollection(edgesList);
	}

	@Override
	public Collection<Node> getIncidentVertices(Edge edge) {
		
		Collection<Node> nodes = new ArrayList<Node>(2);
		nodes.add(edge.getSource());
		nodes.add(edge.getTarget());
		return nodes;
	}

	@Override
	public int getNeighborCount(Node node) {
		int[] list = graph.neighborsArray(graph.getIndex(node));
		if(list == null)
			return 0;
		return list.length;
	}

	@Override
	public Collection<Node> getNeighbors(Node node) {
		int[] list = graph.neighborsArray(graph.getIndex(node));
		if(list == null)
			return null;

		return intArrayToNodeCollection(list);
	}

	@Override
	public int getVertexCount() {
		return graph.getNodeCount();
	}

	@Override
	public Collection<Node> getVertices() {
		List<Node> list = graph.nodesList();
		return listToNodeCollection(list);
	}

	@Override
	public boolean isIncident(Node node, Edge edge) {
		if(edge.getTarget().equals(node))
			return true;
		if(edge.getSource().equals(node) && !edge.isDirected())
			return true;
		return false;
	}

	@Override
	public boolean isNeighbor(Node node1, Node node2) {
		return graph.isNeighbor(node1, node2);
	}

	@Override
	public boolean removeEdge(Edge edge) {

		if(!graph.containsEdge(edge))
			return false;
		graph.removeEdge(graph.getIndex(edge), false);
		return true;
	}

	@Override
	public boolean removeVertex(Node node) {
		if(!graph.containsNode(node))
			return false;
		graph.removeNode(graph.getIndex(node), false);
		return true;
	}
	
	private Collection<Edge> intArrayToEdgeCollection(int[] array){
		
		Collection<Edge> edgeCollection = new ArrayList<Edge>(array.length);
		Edge tempEdge;
		for(int i=0;i<array.length;i++){
			tempEdge = graph.getEdge(array[i]);
			if(tempEdge != null)
				edgeCollection.add(tempEdge);
		}
		return edgeCollection;
	}

	private Collection<Node> intArrayToNodeCollection(int[] array){
		
		Collection<Node> nodeCollection = new ArrayList<Node>(array.length);
		Node tempNode;
		for(int i=0;i<array.length;i++){
			tempNode = graph.getNode(array[i]);
			if(tempNode != null)
				nodeCollection.add(tempNode);
		}
		return nodeCollection;
		
	}
	
	private Collection<Edge> listToEdgeCollection(List<Edge> edgeList){
		
		Collection<Edge> edges = new ArrayList<Edge>(edgeList.size());
		for(Object edge : edgeList)
			edges.add((Edge) edge);
		return edges;
	}

	private Collection<Node> listToNodeCollection(List<Node> nodeList){
		
		Collection<Node> nodes = new ArrayList<Node>(nodeList.size());
		for(Object node : nodeList)
			nodes.add((Node) node);
		return nodes;
	}

	@Override
	public EdgeType getDefaultEdgeType() {

		return EdgeType.DIRECTED;
	}
}
