package plugin;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.ErrorMsg;
import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;

import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.graph.util.Pair;

/**
 * 
 * Wraps an org.graffiti.graph.Graph network to implement the interface Graph<V,E> from JUNG.
 * 
 * @author Johannes Graessler
 */

public class VantedNetworkWrapper<V extends Node,E extends Edge> implements edu.uci.ics.jung.graph.Graph<Node,Edge> {

	protected Graph graph;
	
	/*public VantedNetworkWrapper(){
		graph = new AdjListGraph();
	}*/
	
	public VantedNetworkWrapper(Graph graph){
		this.graph = graph;
	}
	
	@Override
	public boolean addEdge(Edge edge, Node source, Node target) {
		boolean directed = true;

		if(!graph.isDirected())
			directed = false;

		try{
			graph.addEdge(source,target,directed);
		}
		catch(Exception e){
			ErrorMsg.addErrorMessage(e);
			return false;
		}
		
		return true;
	}

	@Override
	public boolean addEdge(Edge edge, Node source, Node target, EdgeType type) {
		boolean directed = true;

		if(type.equals(EdgeType.UNDIRECTED))
			directed = false;
		try{
			graph.addEdge(source,target,directed);
		}
		catch(Exception e){
			ErrorMsg.addErrorMessage(e);
			return false;
		}
		
		return true;
	}

	@Override
	public Node getDest(Edge edge) {
		return edge.getTarget();
	}

	@Override
	public Pair<Node> getEndpoints(Edge edge) {
		Pair<Node> pair = new Pair<Node>(edge.getSource(),edge.getTarget());
		return pair;
	}

	@Override
	public Collection<Edge> getInEdges(Node node) {
		return node.getAllInEdges();
	}

	@Override
	public Node getOpposite(Node node, Edge edge) {
		return edge.getTarget()==node ? edge.getSource() : edge.getTarget();
	}

	@Override
	public Collection<Edge> getOutEdges(Node node) {
		return node.getAllOutEdges();
	}

	@Override
	public int getPredecessorCount(Node node) {
		return node.getAllInNeighbors().size();
	}

	@Override
	public Collection<Node> getPredecessors(Node node) {
		return node.getAllInNeighbors();
	}

	@Override
	public Node getSource(Edge edge) {
		return edge.getSource();
	}

	@Override
	public int getSuccessorCount(Node node) {
		return node.getAllOutNeighbors().size();
	}

	@Override
	public Collection<Node> getSuccessors(Node node) {
		return node.getAllOutNeighbors();
	}

	@Override
	public int inDegree(Node node) {
		return node.getInDegree();
	}

	@Override
	public boolean isDest(Node node, Edge edge) {
		return edge.getTarget()==node ? true : false;
	}

	@Override
	public boolean isPredecessor(Node source, Node target) {
		return target.getAllInNeighbors().contains(source);
	}

	@Override
	public boolean isSource(Node node, Edge edge) {
		return edge.getSource()==node ? true : false;
	}

	@Override
	public boolean isSuccessor(Node target, Node source) {
		return source.getAllOutNeighbors().contains(target);
	}

	@Override
	public int outDegree(Node node) {
		return node.getOutDegree();
	}

	@Override
	public boolean addEdge(Edge edge, Collection<? extends Node> nodes) {
		if(edge.isDirected())
			return this.addEdge(edge, nodes, EdgeType.DIRECTED);
		else
			return this.addEdge(edge, nodes, EdgeType.UNDIRECTED);
	}

	@Override
	public boolean addEdge(Edge edge, Collection<? extends Node> nodes,
			EdgeType edgetype) {
		
		if(nodes.size() != 2)
			return false;
		
		Iterator<? extends Node> it = nodes.iterator();
		return this.addEdge(edge, it.next(),it.next(),edgetype);
	}

	@Override
	public boolean addVertex(Node node) {
		graph.addNodeCopy(node);
		return true;
	}

	@Override
	public boolean containsEdge(Edge edge) {
		return graph.containsEdge(edge);
	}

	@Override
	public boolean containsVertex(Node node) {
		return graph.containsNode(node);
	}

	@Override
	public int degree(Node node) {
		return node.getDegree();
	}

	@Override
	public Edge findEdge(Node source, Node target) {
		//returns the first edge that connects source and target
		return graph.getEdges(source, target).iterator().next();
	}

	@Override
	public Collection<Edge> findEdgeSet(Node source, Node target) {
		return graph.getEdges(source, target);
	}

	@Override
	public int getEdgeCount() {
		return graph.getNumberOfEdges();
	}

	@Override
	public int getEdgeCount(EdgeType edgetype) {
		if(edgetype == EdgeType.DIRECTED)
			return graph.getNumberOfDirectedEdges();
		else
			return graph.getNumberOfUndirectedEdges();
		
	}

	@Override
	public EdgeType getEdgeType(Edge edge) {
		if(edge.isDirected())
			return EdgeType.DIRECTED;
		else
			return EdgeType.UNDIRECTED;
	}

	@Override
	public Collection<Edge> getEdges() {
		return graph.getEdges();
	}

	@Override
	public Collection<Edge> getEdges(EdgeType edgetype) {
		if(edgetype == EdgeType.DIRECTED){
			
			Collection<Edge> edges = new ArrayList<Edge>();
			for(Edge edge : graph.getEdges())
				if(edge.isDirected())
					edges.add(edge);
			return edges;
			//if the graph is directed, we can be sure, that all edges are directed
//			if(graph.isDirected())
//				return graph.getEdges();
		}
		
		//if undirected edges should be returned
		Collection<Edge> edges = new ArrayList<Edge>();
		for(Edge edge : graph.getEdges())
			if(!edge.isDirected())
				edges.add(edge);
		return edges;
	}

	@Override
	public int getIncidentCount(Edge edge) {
		if(edge.getSource() == edge.getTarget())
			return 1;
		return 2;
	}

	@Override
	public Collection<Edge> getIncidentEdges(Node node) {
		return node.getEdges();
	}

	@Override
	public Collection<Node> getIncidentVertices(Edge edge) {
		Collection<Node> nodes = new ArrayList<Node>();
		nodes.add(edge.getSource());
		nodes.add(edge.getTarget());
		return nodes;
	}

	@Override
	public int getNeighborCount(Node node) {
		return node.getNeighbors().size();
	}

	@Override
	public Collection<Node> getNeighbors(Node node) {
		return node.getNeighbors();
	}

	@Override
	public int getVertexCount() {
		return graph.getNumberOfNodes();
	}

	@Override
	public Collection<Node> getVertices() {
		return graph.getNodes();
	}

	@Override
	public boolean isIncident(Node node, Edge edge) {
		if(edge.getSource() == node || edge.getTarget() == node)
			return true;
		return false;
	}

	@Override
	public boolean isNeighbor(Node source, Node target) {
		return source.getNeighbors().contains(target);
	}

	@Override
	public boolean removeEdge(Edge edge) {
		graph.deleteEdge(edge);
		return true;
	}

	@Override
	public boolean removeVertex(Node node) {
		graph.deleteNode(node);
		return true;
	}

	@Override
	public EdgeType getDefaultEdgeType() {
		if(graph.isDirected())
			return EdgeType.DIRECTED;
		return EdgeType.UNDIRECTED;
	}
}
