package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.SparseDoubleMatrix2D;
import cern.jet.math.Functions;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Base class for both current flow centralities.
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 * @param <V>
 * @param <E>
 */
abstract class AbstractCurrentFlowCentrality<V,E> extends VertexCentrality<V,E>{

    /**
     * Creates a "reduced" laplacian matrix for the graph. The term "reduced"
     * means that the zeroth row and column of the matrix is not included. The
     * resulting matrix is a (n-1) x (n-1) matrix!
     * 
     * @return a (n-1) x (n-1) matrix
     */
	
	protected Map<E, Number> edgeWeights = null;
	
	
	public AbstractCurrentFlowCentrality(String name, Graph<V,E> graph){
		super(name,graph);
	}
	
	public AbstractCurrentFlowCentrality(String name, Graph<V,E> graph, Map<E,Number> edgeWeights){
		super(name, graph);
		this.edgeWeights = edgeWeights;
	}
	
    protected DoubleMatrix2D createReducedLaplacian() {

        int numVertices = graph.getVertexCount();

        // L = D - A
        SparseDoubleMatrix2D adjMatrix = GraphMatrixOperations.graphToSparseMatrix(graph, edgeWeights);
        SparseDoubleMatrix2D degreeMatrix = GraphMatrixOperations.createVertexDegreeDiagonalMatrix(graph);
        DoubleMatrix2D laplacian = degreeMatrix.copy();

        laplacian.assign(adjMatrix, Functions.minus);

        DoubleMatrix2D reducedLaplacian = new SparseDoubleMatrix2D(numVertices - 1, numVertices - 1);
        // Skip the first row and column (row==0 and col==0!)
        for (int row = 1; row < laplacian.rows(); row++) {
            for (int col = 1; col < laplacian.columns(); col++) {
                reducedLaplacian.set(row - 1, col - 1, laplacian.get(row, col));
            }
        }
        return reducedLaplacian;
    }

    /**
     * Sets the rankings values for this ranker based on a vector. Usually
     * <code>setRankScore</code> is called for each vertex during the
     * computation but for matrix/vector based operations the outcome is usually
     * a vector not a value for a single element. Therefore this method uses the
     * indexer to find the corresponding vertex for the value of interest.
     * 
     * @param resultVector
     *            the vector
     */
    protected void setRankings(double[] resultVector) {
        BidiMap<V,Integer> idx = Indexer.create(graph.getVertices());

        for (V v : graph.getVertices()){
        	int i = idx.get(v);
            output.put(v, resultVector[i]);
        }
    }

}