package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the Radiality centrality based on partial dependencies introduced by Ulrik Brandes.
 * Before using this class, the preconditions for this centrality should be checked.
 * Otherwise the values may not be correct.
 * 
 * example for usage:
 * 	if(CentralityHandler.isPossible(graph,CentralityHandler.RADIALITY)){
 *  	BrandesRadiality rad = new BrandesRadiality<Vertex,Edge>(graph);
 *  	BrandesSP<Vertex,Edge> bsp = new BrandesSP(graph,edgeWeights);
 *  
 *  	bsp.addListener(rad);
 *  	bsp.start();
 *  }
 * 
 * @author Johannes Graessler
 *
 * @param <V> The type of the vertices
 * @param <E> The type of the edges
 */

public class BrandesRadiality<V, E> extends VertexCentrality<V, E> 
		implements BrandesSPListener<V, E>{

	
	protected double diameter = 0.0;
	
	private boolean finished = false;
	
	
	public BrandesRadiality(Graph<V,E> graph) {
		super(CentralityHandler.RADIALITY, graph);
	}

	@Override
	public void finished() {
		Double tempOut,tempResult;
		int nrNodes = graph.getVertexCount();
		
		for(V v : graph.getVertices()){
			tempOut = output.get(v);
			if(tempOut != null && !tempOut.isInfinite()){
				tempResult = ((diameter+1.0)*nrNodes/(nrNodes-1)) - output.get(v);
				output.put(v, tempResult);
			}
		}
		finished = true;
	}

	@Override
	public void computeDependencies(BrandesSPData<V, E> stepData) {
		
		if(this.graph != stepData.getGraph() || finished)
			return;

		Map<V,BrandesVertexData<E> > vertexData = stepData.getVertexData();
		V source = stepData.getStack().getLast();
		
		if(graph.getVertexCount() != stepData.getStack().size()){
			output.put(source, Double.POSITIVE_INFINITY);
			return;
		}
		
		Double result = 0.0;
		Double dist = 0.0;
		
		for(V target : stepData.getStack()){
			dist = vertexData.get(target).distance;
			//maybe devide dist by (n-1) here, to keep the result small but maybe lose precision
			result += dist;
			if(dist > diameter)
				diameter = dist;
		}
		
		Integer nrNodes = stepData.getStack().size();
		result =  (result/(nrNodes-1.0));
		output.put(source, result);
		
		//helping the garbage collector
		result = null;
		dist = null;
		nrNodes = null;
	}

	@Override
	public Double getVertexScore(V v) {
		if(v == null)
			return null;
		return output.get(v);
	}

}
