package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

import org.apache.commons.collections15.Transformer;
import org.apache.commons.collections15.functors.ConstantTransformer;

import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPData;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesSPListener;
import de.ipk_gatersleben.ag_nw.centilib.utils.BrandesVertexData;
import edu.uci.ics.jung.algorithms.util.MapBinaryHeap;
import edu.uci.ics.jung.graph.Graph;



public class BrandesSP<V,E> {

	protected Graph<V,E> graph;
	protected Map<V, Double> vertex_scores;
	protected Map<E, Double> edge_scores;
	protected Map<V, BrandesVertexData<E> > vertex_data;
	protected Collection<BrandesSPListener<V,E> > listener;
	protected Transformer<E,? extends Number> edgeWeights;
		
	
	public BrandesSP(){};
	
	/**
	 * Calculates betweenness scores based on the all-pairs unweighted shortest paths
	 * in the graph.
	 * @param graph the graph for which the scores are to be calculated
	 */
	@SuppressWarnings("unchecked")
	public BrandesSP(Graph<V,E> graph) {
		setGraph(graph);
		
	}

	/**
	 * Calculates betweenness scores based on the all-pairs weighted shortest paths in the
	 * graph.
	 * 
	 * <p>NOTE: This version of the algorithm may not work correctly on all graphs; we're still
	 * working out the bugs.  Use at your own risk.
	 * @param graph the graph for which the scores are to be calculated
	 * @param edge_weights the edge weights to be used in the path length calculations
	 */
	public BrandesSP(Graph<V, E> graph, 
			Transformer<E, ? extends Number> edge_weights) 
	{
		setGraph(graph,edge_weights);
		
	}

	protected void initialize(Graph<V,E> graph)
	{
		this.graph = graph;
		this.vertex_scores = new HashMap<V, Double>();
		this.edge_scores = new HashMap<E, Double>();
		this.vertex_data = new HashMap<V, BrandesVertexData<E> >();
		this.listener = new HashSet<BrandesSPListener<V,E> >();
	}
	
	@SuppressWarnings("unchecked")
	public void setGraph(Graph<V,E> graph){
		setGraph(graph, new ConstantTransformer(1.0));	
	}
	
	public void setGraph(Graph<V,E> graph, Transformer<E,? extends Number> edgeWeights){
		if(graph == null)
			return;
		
		if(edgeWeights == null)
			this.edgeWeights = new ConstantTransformer(1);
		
		else{
			this.edgeWeights = edgeWeights;			

			// reject negative-weight edges up front
			for (E e : graph.getEdges())
			{
				double e_weight = this.edgeWeights.transform(e).doubleValue();
	        	if (e_weight < 0)
	        		throw new IllegalArgumentException(String.format(
	        				"Weight for edge '%s' is < 0: %d", e, e_weight)); 
			}
		}
		initialize(graph);
	}
	
	public void start(){
		computeShortestPaths(new MapBinaryHeap<V>(new ShortestPathComparator()));
	}
	
	
	
	public void addListener(BrandesSPListener<V,E> listener){
		if(listener == null)
			throw new IllegalArgumentException("listener is null");
		
		this.listener.add(listener);
	}
	
	protected void computeShortestPaths(Queue<V> queue){
		LinkedList<V> stack = new LinkedList<V>();
		
		for (V v : graph.getVertices())
		{
			// initialize the betweenness data for this new vertex
			vertex_data.clear();
			for (V s : graph.getVertices()) 
				this.vertex_data.put(s, new BrandesVertexData<E>());

            vertex_data.get(v).numSPs = 1;
            vertex_data.get(v).distance = 0;

            stack.clear();
            queue.offer(v);

            while (!queue.isEmpty()) 
            {
            	V w = queue.poll();
                stack.push(w);
            	BrandesVertexData<E> w_data = vertex_data.get(w);
                
                for (E e : graph.getOutEdges(w))
                {
                	V x = graph.getOpposite(w, e);
                	if (x.equals(w))
                		continue;
                	double wx_weight = edgeWeights.transform(e).doubleValue();
                	
                	BrandesVertexData<E> x_data = vertex_data.get(x);
                	double x_potential_dist = w_data.distance + wx_weight;
                	
                    if (x_data.distance < 0) 
                    {
                    	x_data.distance = x_potential_dist;
                      	queue.offer(x);
                    }
                    
                    // note:
                    // (1) this can only happen with weighted edges
                    // (2) x's SP count and incoming edges are updated below 
                    if (x_data.distance > x_potential_dist)
                    {
                    	x_data.distance = x_potential_dist;
                    	// invalidate previously identified incoming edges
                    	// (we have a new shortest path distance to x)
                    	x_data.incomingEdges.clear();
                    	
                    	// ***** von mir eingefuegt
                    	x_data.numSPs = 0;
                    	
                        // update x's position in queue
                    	((MapBinaryHeap<V>)queue).update(x);
                    }
                }
               
                for (E e: graph.getOutEdges(w))
                {
                	V x = graph.getOpposite(w, e);
                	
                	if (x.equals(w))
                		continue;
                	
                	double e_weight = edgeWeights.transform(e).doubleValue();
                	BrandesVertexData<E> x_data = vertex_data.get(x);
                	double x_potential_dist = w_data.distance + e_weight;
                   
                	if (x_data.distance == x_potential_dist) 
                    {
                        x_data.numSPs += w_data.numSPs;
//                        vertex_data.get(x).predecessors.add(w);
                        x_data.incomingEdges.add(e);
                    }
                }
            }
            finishedSource(stack);
		}

		for(BrandesSPListener<V,E> l : listener)
			l.finished();
		
		listener.clear();
        vertex_data.clear();
	}
	
	private void finishedSource(LinkedList<V> stack){
		LinkedList<V> tempStack = new LinkedList<V>(stack);
		BrandesSPData<V,E> data = new BrandesSPData<V,E>();
		
		data.setGraph(this.graph);
		data.setVertexData(Collections.unmodifiableMap(vertex_data));
		
		for(BrandesSPListener<V,E> listener : this.listener){

			if(tempStack.size() != stack.size())
				tempStack = new LinkedList<V>(stack);

			data.setStack(tempStack);
			listener.computeDependencies(data);
		}
	}

	private class ShortestPathComparator implements Comparator<V>
    {
		public int compare(V v1, V v2) 
		{
			if(vertex_data.get(v1).distance == vertex_data.get(v2).distance)
				return 0;
			
			return vertex_data.get(v1).distance > vertex_data.get(v2).distance ? 1 : -1;
		}
    }

}
