package de.ipk_gatersleben.ag_nw.centilib.centralities;

import org.apache.commons.collections15.BidiMap;

import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;


/**
 * Algorithm for the centrality "Centroid Values". 
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
public class CentroidValues<V,E> extends VertexCentrality<V,E>{

    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.CentroidValues.RankScore";

    protected Distance<V> distance; 
    
    int numVertices;
    

    public CentroidValues(Graph<V,E> graph) {
        this(graph,new UnweightedShortestPath<V, E>(graph));
    }
    
    public CentroidValues(Graph<V,E> graph, Distance<V> distance){
    	super("Centroid",graph);
    	
        numVertices = graph.getVertexCount();
    	this.distance = distance;
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public Double getVertexScore(V v) {

    	Double value = output.get(v);
	    if (value != null)
	        return value;

        BidiMap<V,Integer> indexer = Indexer.create(graph.getVertices());

        int[][] gammaValues = createEmptyMatrix();
        computeGammaValues (indexer, gammaValues);
        int[][] fValues = computeFValue(gammaValues);


        for (V w : graph.getVertices()){
            int i = indexer.get(w);
            output.put(w, extractMinFValue(fValues, i));
        }
        
        return output.get(v);
    }

    private int[][] createEmptyMatrix() {
        int[][] gammaValues = new int[numVertices][numVertices];
        for (int i = 0; i < numVertices; i++) {
            for (int j = 0; j < numVertices; j++) {
                gammaValues[i][j] = 0;
            }
        }
        return gammaValues;
    }

    private void computeGammaValues(BidiMap<V,Integer> vertexIndexer, int[][] gammaValues) {
        for (int u = 0; u < numVertices; u++) {
            for (int v = 0; v < numVertices; v++) {
                for (int w = 0; w < numVertices; w++) {
                    Number distUW = distance.getDistance((V) vertexIndexer.getKey(u), (V) vertexIndexer.getKey(w));
                    Number distVW = distance.getDistance((V) vertexIndexer.getKey(v), (V) vertexIndexer.getKey(w));
                    if (distUW == null || distVW == null) {
                        throw new IllegalArgumentException("Graph is not strong connected");
                    }
                    if (distUW.intValue() < distVW.intValue()) {
                        gammaValues[u][v]++;
                    }
                }
            }
        }
    }

    private int[][] computeFValue(int[][] gammaValues) {
        int[][] f = new int[numVertices][numVertices];
        for (int u = 0; u < numVertices; u++) {
            for (int v = 0; v < numVertices; v++) {
                f[u][v] = gammaValues[u][v] - gammaValues[v][u];
            }
        }
        return f;
    }

    private Double extractMinFValue(int[][] f, int u) {
        Double result = Double.MAX_VALUE;
        for (int i = 0; i < numVertices; i++) {
            if (f[u][i] < result && u != i) {
                result = Double.valueOf(Integer.toString(f[u][i]));
            }
        }
        return result;
    }
}