package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.Pair;

/**
 * Computes the current flow betweenness centrality for the given graph. This
 * implementation is based on Daniel Fleischer's implementation for yFiles and
 * JMP. Thanks to Daniel for providing his implementation! <br>
 * <em>Beware: My implementation is not optimized in any sense.
 *     Especially the matrix operations are not checked for 
 *     the fastes algorithm.
 * </em>
 * 
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
public class CurrentFlowBetweenness<V,E> extends AbstractCurrentFlowCentrality<V,E> {

    private final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.CurrentFlowBetweenness";

    /**
     * The Constructor for the CF Betweenness Ranker.
     * 
     * @param g
     *            A connected, simple, loopfree and undirected graph
     */
    public CurrentFlowBetweenness(Graph<V,E> g) {
    	super("CurrentFlowBetweenness",g);
    }
    
    public CurrentFlowBetweenness(Graph<V,E> graph, Map<E, Number> edgeWeights){
    	super("CurrentFlowBetweenness",graph,edgeWeights);
    }

    /**
     * Returns the user data key for this ranker.
     */
    public String getRankScoreKey() {
        return KEY;
    }

    /**
     * Computes the rankings.
     */
    public Double getVertexScore(V v) {

    	Double value = output.get(v);
	    if (value != null)
	        return value;

        // The following code is copied from Daniels implementation and only
        // adapted in places where Y and JUNG and COLT and JMP differ.
    	
        Collection<E> allEdges = graph.getEdges();
        int numVertices = graph.getVertexCount();
        double[] resultVector = new double[numVertices];
        double[] A = new double[numVertices];
        int[] pos = new int[numVertices];
        int[] pos_inv = new int[numVertices];

        DoubleMatrix2D reducedLaplacian = createReducedLaplacian();
        DoubleMatrix2D C = createInverseC(numVertices, reducedLaplacian);

        for (int i = 0; i < numVertices; i++) {
            resultVector[i] = 0;
        }

        BidiMap<Integer,V> idx = Indexer.create(graph.getVertices()).inverseBidiMap();
        for (Iterator<E> it = allEdges.iterator(); it.hasNext();) {
            E edge = (E) it.next();
            Pair<V> endpoints = graph.getEndpoints(edge);
            int sourceId = idx.getKey((V) endpoints.getFirst());
            int targetId = idx.getKey((V) endpoints.getSecond());

            for (int j = 0; j < numVertices; j++) {
                A[j] = C.get(sourceId, j) - C.get(targetId, j);
                pos_inv[j] = j;
            }
            mergeSort(A, pos_inv, 0, numVertices - 1);
            for (int j = 0; j < numVertices; j++) {
                pos[pos_inv[j]] = j;
            }

            for (int j = 0; j < numVertices; j++) {
                resultVector[targetId] += (numVertices - 1 - pos[j] - j) * A[pos[j]];
                resultVector[sourceId] += (j - pos[j]) * A[pos[j]];
            }
        }

        // necessary because above only current into vertices
        // and not vice versa is measured
        for (int i = 0; i < numVertices; i++) {
            resultVector[i] = 2 * (resultVector[i] - i) / (numVertices - 1) / (numVertices - 2);

            // This line leads to the results from the newman paper.  See the mail from Daniel Fleischer
            // resultVector[i] = (2 * (resultVector[i] - i + numVertices-1)) / ((numVertices) * (numVertices - 1));            
        }

        // normalize ??
        // Commented in Brandes Fleischer, too!
        // for(int i=0;i<N;i++) x[i]/=N*(N-1)/2;

        setRankings(resultVector);
        return output.get(v);
    }

    // Daniels implementation used the conjugate gradient method to calculate
    // the inverse of a matrix.
    // afaik this is not available in COLT, therefore this is my implementation
    // based on COLTs algorithms.
    private DoubleMatrix2D createInverseC(int numVertices, DoubleMatrix2D reducedLaplacian) {

        Algebra algebra = new Algebra();
        DoubleMatrix2D reducedC = algebra.inverse(reducedLaplacian);
        DoubleMatrix2D C = new DenseDoubleMatrix2D(numVertices, numVertices);

        // Copy the rows and cols 1..N
        for (int row = 0; row < reducedC.rows(); row++) {
            for (int col = 0; col < reducedC.columns(); col++) {
                C.set(row + 1, col + 1, reducedC.get(row, col));
            }
        }

        // Initialize row and col 0
        for (int i = 0; i < numVertices; i++) {
            C.set(0, i, 0);
            C.set(i, 0, 0);
        }

        return C;
    }

    // Daniels implementation of merge sort
    private static void mergeSort(double[] A, int[] pos, int p, int r) {
        int q;

        if (r <= p) {
            return;
        }
        q = Math.round((p + r) / 2);
        mergeSort(A, pos, p, q);
        mergeSort(A, pos, q + 1, r);
        merge(A, pos, p, q, r);
    }

    // Daniels implementation of merge
    private static void merge(double[] A, int[] pos_old, int p, int q, int r) {
        int i, j, m;
        int pos[] = new int[r + 1];
        double[] B = new double[r + 1];

        i = p;
        j = q + 1;
        m = p;
        while (i <= q && j <= r) {
            if (A[i] > A[j]) {
                pos[m] = pos_old[i];
                B[m] = A[i++];
            } else {
                pos[m] = pos_old[j];
                B[m] = A[j++];
            }
            m++;
        }
        if (i > q) {
            for (int l = j; l <= r; l++) {
                pos[m + l - j] = pos_old[l];
                B[m + l - j] = A[l];
            }
        } else {
            for (int l = i; l <= q; l++) {
                pos[m + l - i] = pos_old[l];
                B[m + l - i] = A[l];
            }
        }
        for (int l = p; l <= r; l++) {
            pos_old[l] = pos[l];
            A[l] = B[l];
        }
    }
}