package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.LUDecomposition;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the current flow closeness centrality for the given graph. This
 * implementation is based on Daniel Fleischer's implementation for yFiles and
 * JMP. Thanks to Daniel for providing his implementation! <br>
 * <em>Beware: My implementation is not optimized in any sense.
 *     Especially the matrix operations are not checked for 
 *     the fastes algorithm.
 * </em>
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 */
public class CurrentFlowCloseness<V,E> extends AbstractCurrentFlowCentrality<V,E> {

    private final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.CurrentFlowCloseness";

    /**
     * The Constructor for the CF Closeness Ranker.
     * 
     * @param g
     *            A connected, simple, loopfree and undirected graph
     */
    public CurrentFlowCloseness(Graph<V,E> g) {
    	super("CurrentFlowCloseness",g);
    }
    
    public CurrentFlowCloseness(Graph<V,E> graph, Map<E, Number> edgeWeights){
    	super("CurrentFlowCloseness",graph,edgeWeights);
    }

    /**
     * Returns the user data key for this ranker.
     */
    public String getRankScoreKey() {
        return KEY;
    }

    /**
     * Computes the rankings.
     */
    public Double getVertexScore(V v) {

    	Double value = output.get(v);
	    if (value != null)
	        return value;

        int numVertices = graph.getVertexCount();
        double[] resultVector = new double[numVertices];
        DoubleMatrix2D solveTemporaryVector = new DenseDoubleMatrix2D(numVertices - 1, 1);
        DoubleMatrix2D kroneckerVector = new DenseDoubleMatrix2D(numVertices - 1, 1);
        // Algebra algebra = new Algebra();

        DoubleMatrix2D reducedLaplacian = createReducedLaplacian();
        LUDecomposition lud = new LUDecomposition(reducedLaplacian);

        // Initialize result vector
        for (int i = 0; i < numVertices; i++) {
            resultVector[i] = 0;
        }

        // Loop over every vertex to compute the rank score
        for (int i = 0; i < numVertices - 1; i++) {

            // Cleanup the temporary and initialize the kronecker vector
            for (int j = 0; j < numVertices - 1; j++) {
                solveTemporaryVector.set(j, 0, 0);
                if (j == i)
                    kroneckerVector.set(j, 0, 1);
                else
                    kroneckerVector.set(j, 0, 0);
            }

            // This may be a potential place for optimization!
            // We solve the linear equation system from scratch for many times!
            // solveTemporaryVector = algebra.solve(reducedLaplacian, kroneckerVector);
            solveTemporaryVector = lud.solve(kroneckerVector);

            // Compute the centrality values
            resultVector[0] += solveTemporaryVector.get(i, 0);
            resultVector[i + 1] += solveTemporaryVector.get(i, 0);

            for (int j = 0; j < numVertices - 1; j++) {
                resultVector[i + 1] += solveTemporaryVector.get(i, 0) - 2 * solveTemporaryVector.get(j, 0);
                resultVector[j + 1] += solveTemporaryVector.get(i, 0);
            }
        }

        // We need the reciprocal, because the computation leads to the
        // replacement resistances and we need the conductance ("leitwert")
        for (int i = 0; i < numVertices; i++) {
            // (numVertices-1) is n_c as defined on page 536 mid.
            resultVector[i] = (numVertices - 1) / resultVector[i];
        }

        setRankings(resultVector);
        return output.get(v);
    }
}
