package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.SparseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import cern.jet.math.Functions;
import cern.jet.math.Mult;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;


/**
 * A node importance ranker based on the hubbell index.
 * 
 * @author Johannes Graessler
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
public class HubbellIndex<V,E> extends VertexCentrality<V,E> {
    
    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.HubbellIndex.RankScore";

    private SparseDoubleMatrix2D weightedAdjacencyMatrix;
    
    protected boolean isSolvable = false;
    
    
    
    public HubbellIndex(Graph<V,E> graph, double weightFactor) {
    	this(graph,null,weightFactor,isSolvable(graph,null,weightFactor));
    }
    
    public HubbellIndex(Graph<V,E> graph, Map<E,Number> edgeWeights, double weightFactor){
    	this(graph,edgeWeights,weightFactor,isSolvable(graph,edgeWeights,weightFactor));
    }
    
    /**
     * Only use this constructor if you are sure that the HubbellIndex
     * is solvable for the given graph and weightFactor (call the static method
     * isSolvable(graph,weightFactor) first)
     * 
     */
    public HubbellIndex(Graph<V,E> graph, Map<E,Number> edgeWeights, double weightFactor, boolean isSolvable){
    	super("HubbelIndex",graph);
    	weightedAdjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph, edgeWeights);
        weightedAdjacencyMatrix.assign(Mult.mult(weightFactor));
        this.isSolvable = isSolvable;
    	
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public boolean isSolvable() {

    	return isSolvable;
    }
    
    public static <V,E> boolean isSolvable(Graph<V,E> graph, Map<E,Number> edgeWeights, Double weightFactor){
    	
    	if(weightFactor == null) return false;
    	
    	SparseDoubleMatrix2D weightedAdjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph,edgeWeights);
    	weightedAdjacencyMatrix.assign(Mult.mult(weightFactor));
    	
        EigenvalueDecomposition ed = new EigenvalueDecomposition(weightedAdjacencyMatrix);
        DoubleMatrix1D realEigenvalues = ed.getRealEigenvalues();
        
        for (int i = 0; i < realEigenvalues.size(); i++) {
            if (realEigenvalues.get(i) > 1.0) {
                return false;
            }
        }
        
        return true;
    }

    public Double getVertexScore(V v){

    	Double value = output.get(v);
	    if (value != null)
	        return value;
    	
    	if (!isSolvable) {
            return Double.NaN;
        }

        // 
        //
        //
        // Beware: Colt works a little bit strange with the matrices! 
        // Read the documentation!
        //
        //
        //

        Algebra alg = new Algebra();

        // The auxilary information "E". 
        // We use the vector of ones as our default.
        // TODO: Realise the vector E in the hubbell index.        
        DoubleMatrix2D vectorE = DoubleFactory2D.dense.make(weightedAdjacencyMatrix.rows(), 1, 1);
        
                // The row vector containing zeros.
        DoubleMatrix2D res = DoubleFactory2D.dense.make(weightedAdjacencyMatrix.rows(), 1, 0);

        // The matrix with ones on the main diagonal
        DoubleMatrix2D identityMatrix = DoubleFactory2D.sparse.identity(weightedAdjacencyMatrix.rows());
        
        // Change I towards I-W
        identityMatrix.assign(weightedAdjacencyMatrix, Functions.minus);
        
        // Now compute the inverse...
        DoubleMatrix2D inverseIminusW = alg.inverse(identityMatrix);
        
        // res = ((I-W)^-1)*vectorE
        inverseIminusW.zMult(vectorE, res);
        
        // Move everything to the vertices.
        BidiMap<V,Integer> indexer = Indexer.create(graph.getVertices());
        for (V w : graph.getVertices()) {
            int i = indexer.get(w);
            output.put(w, res.get(i, 0));
        }

        return output.get(v);
    }

}