package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.text.DecimalFormat;
import java.util.Map;

import org.apache.commons.collections15.BidiMap;

import cern.colt.matrix.DoubleFactory2D;
import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import cern.jet.math.Functions;
import cern.jet.math.Mult;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * A node importance ranker based on the counting all Paths algorithm.
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
public class KatzStatusIndex<V,E> extends VertexCentrality<V,E> {

    public final static String KEY = "de.ipk_gatersleben.ag_nw.centilib.centralities.CountingAllPaths.RankScore";

    private double alpha;

    private DoubleMatrix2D adjacencyMatrix;
    
    public KatzStatusIndex(Graph<V,E> graph, double alpha) {
    	this(graph,null,alpha);
    }
    
    public KatzStatusIndex(Graph<V,E> graph, Map<E,Number> edgeWeights, double alpha){
    	super("KatzStatus",graph);
    	this.alpha = alpha;
        if(edgeWeights != null)
        	adjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph, edgeWeights);
        else
        	adjacencyMatrix = GraphMatrixOperations.graphToSparseMatrix(graph);
        
        DecimalFormat df = new DecimalFormat("#.##");
        this.name += df.format(alpha);
    }

    public String getRankScoreKey() {
        return KEY;
    }

    public static <V,E> double getMaxAlphaValue(Graph<V,E> graph) {
        DoubleMatrix2D aM = GraphMatrixOperations.graphToSparseMatrix(graph, null);
        return getMaxAlphaValue(aM);
    }

    public static double getMaxAlphaValue(DoubleMatrix2D aM) {
        EigenvalueDecomposition ed = new EigenvalueDecomposition(aM);
        return (1 / findMaxEigenvalue(ed.getRealEigenvalues()));
    }
    
    public Double getVertexScore(V v) throws IllegalArgumentException {

    	Double value = output.get(v);
	    if (value != null)
	        return value;

    	if (alpha <= 0 || alpha >= getMaxAlphaValue(adjacencyMatrix)) {
            return Double.NaN;
        }

        // 
        //
        //
        // Beware: Colt works a little bit strange with the matrices! 
        // Read the documentation!
        //
        //
        //

        DoubleMatrix2D adjacencyMatrixT;
        DoubleMatrix2D identityMatrix;
        DoubleMatrix2D inverseMatrix;
        DoubleMatrix2D ones;
        DoubleMatrix2D res;

        Algebra alg = new Algebra();

        // transposed adj matrix.
        adjacencyMatrixT = alg.transpose(adjacencyMatrix);

        // diagonal matrix with 1 one the main diagonal.
        identityMatrix = DoubleFactory2D.sparse.identity(adjacencyMatrix.rows());

        // a row vector with ones.
        ones = DoubleFactory2D.dense.make(adjacencyMatrix.rows(), 1, 1);

        // Identity matrix contains now the result of the equation:
        // identityMatrix = (identityMatrix - alpha*adjT)
        // This operation destroys the identity matrix and the transposed adj matrix!!
        identityMatrix.assign(adjacencyMatrixT.assign(Mult.mult(alpha)), Functions.minus);

        // a row vector of zeros.
        res = DoubleFactory2D.dense.make(adjacencyMatrix.rows(), 1, 0);

        // Compute the inverse for the formula above.
        inverseMatrix = alg.inverse(identityMatrix);

        // multiply the inverted result with the row vector of ones.
        // the result is finally available in the vector res.
        inverseMatrix.zMult(ones, res);

        // Move the result from the vector res to the vertices. 
        BidiMap<V,Integer> indexer= Indexer.create(graph.getVertices());
        for (V w : graph.getVertices()){
            int i = indexer.get(w);
            output.put(w, res.get(i, 0));
        }
        
        return output.get(v);
    }

    private static double findMaxEigenvalue(DoubleMatrix1D realEigenvalues) {
        double currentMax = Double.NEGATIVE_INFINITY;
        for (int i = 0; i < realEigenvalues.size(); i++) {
            if (currentMax < realEigenvalues.get(i)) {
                currentMax = realEigenvalues.get(i);
            }
        }
        return currentMax;
    }
}