package de.ipk_gatersleben.ag_nw.centilib.centralities;

import java.util.HashMap;
import java.util.Map;

import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * This abstract class implements the interface VertexScorer<V, Double> and adds some methods
 * for accessing the name, graph and average centrality value.
 *   
 * @author Johannes Graessler
 *
 * @param <V> Type of nodes
 * @param <E> Type of edges
 */
public abstract class VertexCentrality<V,E> implements VertexScorer<V, Double> {

	protected String name = "";
	
	protected Map<V,Double> output = null;
	
	protected Graph<V,E> graph = null;
	
	protected Double average = Double.NaN; 
	
	
	/**
	 * Create an instance for the given graph.
	 * 
	 * @param name The name of the centrality.
	 * @param graph The graph this centrality is computed for.
	 */
	public VertexCentrality(String name, Graph<V,E> graph){
		this.name = name;
		this.graph = graph;
		this.output = new HashMap<V,Double>(graph.getVertexCount());
	}
	
	/**
	 * Returns the name of this centrality.
	 * 
	 * @return the name of the centrality.
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * Returns the graph, this centrality is computed for.
	 * @return the graph this centrality is computed for.
	 */
	public Graph<V,E> getGraph(){
		return graph;
	}
	
	/**
	 * Returns the average value of this centrality if possible. If the values were not computed yet or there are some values
	 * missing Double.NaN is returned. 
	 * @return the average value or Double.NaN
	 */
	public Double getAverageValue(){
		if(!average.isNaN())
			return average;
		
		if(output == null || output.size() != graph.getVertexCount())
			return Double.NaN;

		Double sum = 0.0;
		Double value = null;
		for(V v : graph.getVertices()){
			value = output.get(v);
			
			//if there is a vertex without a value, something is wrong and we'll quit
			if(value == null)
				return Double.NaN;
			sum += value;
		}
		average = sum / graph.getVertexCount();
		return average;			
	}
}
