package de.ipk_gatersleben.ag_nw.centilib.gui;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.table.AbstractTableModel;

import org.apache.commons.collections15.BidiMap;
import org.apache.commons.collections15.Transformer;

import cern.colt.Sorting;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.algorithms.util.Indexer;
import edu.uci.ics.jung.graph.Graph;

/**
 * The table model for the visualisation of the rankings. 
 *
 * @author Johannes Graessler
 * @author Dirk Koschuetzki
 */
@SuppressWarnings("serial")
public class RankerTableModel<V,E> extends AbstractTableModel {

    protected VertexCentrality<V,E> ranker;
    
    protected Transformer<V,String> transformer; 

    protected BidiMap<V,Integer> output = Indexer.create(new ArrayList<V>());
    
    public RankerTableModel(VertexCentrality<V,E> ranker, Transformer<V,String> vTransformer,Graph<V,E> graph) {
    	
    	super();
        this.ranker = ranker;
        this.transformer = vTransformer;
        if(graph.getVertexCount()>0)
        	calculateSortedScores(graph);
    }
    
    public RankerTableModel(VertexCentrality<V,E> ranker, Transformer<V,String> vTransformer, Collection<V> vertices){

    	super();
        this.ranker = ranker;
        this.transformer = vTransformer;
        if(vertices.size()>0)
        	calculateSortedScores(vertices);
    }

    public int getColumnCount() {
        return 2;
    }

    public int getRowCount() {
        return output.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
    	if (columnIndex == 0) {
            String label = transformer.transform(output.getKey(rowIndex));
            if (label == null || label.equals("")) {
                return new Integer(rowIndex+1);
            }
            return label;
        }

        DecimalFormat df = new DecimalFormat("#.###############");
        
        Double score = ranker.getVertexScore(output.getKey(rowIndex));
        
        if(score == null)
        	return Double.NaN;//.format(0.0);
        else
        	return df.format(score);
    }

    public V getVertexAtPos(int pos) {
        return output.getKey(pos);
    }
    
    public int getPos(V v){
    	if(output.get(v) == null)
    		return -1;
    	return output.get(v);
    }
    
    @Override
    public String getColumnName(int column) {
        assert (column == 0 || column == 1);
        return (column == 0) ? "Vertex" : "Centrality";
    }
    
  
    private void calculateSortedScores(Graph<V,E> graph){
    	//TODO sorting is ok, but mapping the scores to the vertices runs in O(n*logn) again
    	//		perhaps there is a better way
    	
    	if(GraphCachingUtils.getResultList(graph, ranker)!= null){
    		output = GraphCachingUtils.getResultList(graph, ranker);
    		return;
    	}
    	
    	calculateSortedScores(graph.getVertices());
    	GraphCachingUtils.setResultList(graph, ranker, output);
    }
    
    private void calculateSortedScores(Collection<V> vertices){
    	
    	double[] scores = new double[vertices.size()];
    	int i = 0;
    	output.clear();

    	for(V v : vertices){
    		if(ranker.getVertexScore(v)==null)
    			scores[i] = 0;
    		else
    			scores[i] = ranker.getVertexScore(v);
    		i++;
    	}

    	Sorting.mergeSort(scores, 0, scores.length);
    
    	Double score;
    	for(V v : vertices){
    		score = ranker.getVertexScore(v);
    		if (score == null)
    			score = Double.NaN;
    		for(int j = 0;j<scores.length;j++){
    			//for every vertex find the index of his value in the sorted list
    			if(scores[j] == score && output.getKey(scores.length-1-j)==null){
    				//output.getKey(..) has to be null, because different vertices with the same value
    				//must have different indices in the sorted list
    				output.put(v,scores.length-1-j);
    				break;
    			}
    		}
    	}
    		
    }
}