package de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities;

import java.util.Map;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.CentiLibPageRank;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.IterativeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphPreconditionChecks;
import de.ipk_gatersleben.ag_nw.centilib.utils.UniformWeightedOutEdges;
import edu.uci.ics.jung.graph.Graph;

/**
 * Computes the PageRank centralities.
 *
 * @author Johannes Graessler 
 * @author Dirk Koschuetzki
 * @author Michael Telgkamp
 */
@SuppressWarnings("serial")
public class PageRankCentralityAction<V,E> extends IterativeCentralityAction<V,E> {

    public PageRankCentralityAction(CentiLib<V,E> centibin) {
        super(centibin, "*** PageRank");
        scorerName = "PageRank";
    }

    public boolean isPossible() {
        return GraphPreconditionChecks.checkPreconditionsSimple(getCentiLib());
    }

    public boolean compute(Graph<V,E> graph) {

        Double bias = getCentiLib().askForDouble("Please enter a bias value.", "0.15", 0, 1);

        if (bias == null) {
        	scorer = null;
            return false;
        }
        
        scorerName = "PageRank"+bias;
        
        scorer = GraphCachingUtils.getScorer(graph, scorerName);
        if(scorer == null)
        	scorer = calculatePageRank(graph,bias);
        return true;
    }

    public VertexCentrality<V,E> calculatePageRank(Graph<V,E> graph, double bias) {
    	CentiLibPageRank<V, E> pr = new CentiLibPageRank<V, E>(graph,bias);
    	int iterations = 50, more = 0;
    	do{
    		calculatePageRank(pr, iterations);
    		if(pr.getIterations() == pr.getMaxIterations())
    			more = getNumberOfIterations();
    		iterations += more;
    	}while(more > 0);
    	
    	return pr;
    }
    
    public static <V,E> CentiLibPageRank<V,E> calculatePageRank(Graph<V,E> graph, double bias, int iterations, Map<E, Number> edgeWeights){
    	Transformer<E,Double> transformer = null;
    	CentiLibPageRank<V,E> pr = null;

    	try{
    		if(edgeWeights != null)
    			transformer = new UniformWeightedOutEdges<V, E>(graph, edgeWeights);
    	}
    	catch(IllegalArgumentException e){
    		return null;
    	};
    	
    	if(transformer != null)    		
    		pr = new CentiLibPageRank<V,E>(graph,transformer,bias);
    	else
    		pr = new CentiLibPageRank<V,E>(graph,bias);
    	calculatePageRank(pr, iterations);
    	return pr;
    }
    
    private static <V,E> void calculatePageRank(CentiLibPageRank<V,E> pr, int iterations){
    	pr.setMaxIterations(iterations);
    	pr.setTolerance(IterativeCentralityAction.TOLERANCE);
    	pr.evaluate();
    }
}