package de.ipk_gatersleben.ag_nw.centilib.gui.utils;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * Contains methods for recieving Numbers via JOptionPane.
 *
 * @author Johannes Graessler 
 * @author Michael Telgkamp
 * @author Dirk Koschuetzki
 */
public class JGetNumber {

	private final static Object[] OK_CANCEL = { "OK", "Cancel" };

	private final static Object[] YES_NO = { "Yes", "No" };	

	private final static double MAX_DOUBLE = Double.POSITIVE_INFINITY;

	private final static double MIN_DOUBLE = Double.NEGATIVE_INFINITY;

	/**
	 * Opens a JOptionPane asking for an Integer and returns this Integer
	 * 
	 * @param parentFrame:
	 *            Frame to be disabled
	 * @param question:
	 *            Question to be displayed
	 * @param presetValue:
	 *            Preset Value (can be an empty string)
	 * @return the input Value or null
	 */
	public static Integer getInteger(JFrame parentFrame, String question,
			String presetValue) {
		return getInteger(parentFrame, question, presetValue,
				Integer.MIN_VALUE, Integer.MAX_VALUE);
	}

	/**
	 * Opens a JOptionPane asking for an Integer and returns this Integer (with
	 * range for Integer)
	 * 
	 * @param parentFrame:
	 *            Frame to be disabled
	 * @param question:
	 *            Question to be displayed
	 * @param presetValue:
	 *            Preset Value (can be an empty string)
	 * @param minValue:
	 *            minimum Value
	 * @param maxValue:
	 *            maximum Value
	 * @return the input Value or null
	 */
	public static Integer getInteger(JFrame parentFrame, String question1,
			String presetValue, int minValue, int maxValue) {

		boolean hasMax = (maxValue != Integer.MAX_VALUE);
		boolean hasMin = (minValue != Integer.MIN_VALUE);
		String question2 = question1.toString();
        
		if (hasMin && hasMax) {
			question2 += "\n(between " + minValue + " and " + maxValue + ")";
		} else {
			if (hasMin) {
				question2 += "\n(>= " + minValue + ")";
			}
			if (hasMax) {
				question2 += "\n(<= " + maxValue + ")";
			}
		}
		
		String valueString = JOptionPane.showInputDialog(parentFrame,
				question2, presetValue);
		
		if (valueString != null) {
			try {
				Integer value = new Integer(valueString);
				
				if (value.intValue() < minValue || value.intValue() > maxValue) {
					if (JOptionPane.showOptionDialog(parentFrame, " \""
							+ valueString + "\" is not in Range [" + minValue
							+ "," + maxValue + "].\nClick OK to retype",
							"Warning", JOptionPane.DEFAULT_OPTION,
							JOptionPane.WARNING_MESSAGE, null, OK_CANCEL,
							OK_CANCEL[0]) == 0) {
						return getInteger(parentFrame, question1, presetValue,
								minValue, maxValue);
					}
				} else {
					return value;
				}
			} catch (NumberFormatException e) {

				if (JOptionPane.showOptionDialog(parentFrame, " \""
						+ valueString
						+ "\" is not an int number.\nClick OK to retype",
						"Warning", JOptionPane.DEFAULT_OPTION,
						JOptionPane.WARNING_MESSAGE, null, OK_CANCEL, OK_CANCEL[0]) == 0) {
		
					return getInteger(parentFrame, question1, presetValue,
							minValue, maxValue);
					
				}
			}
		}
		return null;
	}

	/**
	 * Opens a JOptionPane asking for a Double and returns this Double
	 * 
	 * @param parentFrame:
	 *            Frame to be disabled
	 * @param question:
	 *            Question to be displayed
	 * @param presetValue:
	 *            Preset Value (can be an empty string)
	 * @return the input Value or null
	 */
	public static Double getDouble(JFrame parentFrame, String question,
			String presetValue) {
		return getDouble(parentFrame, question, presetValue, MIN_DOUBLE,
				MAX_DOUBLE);
	}

	/**
	 * Opens a JOptionPane asking for a Double and returns this Double (with
	 * range for Double)
	 * 
	 * @param parentFrame:
	 *            Frame to be disabled
	 * @param question:
	 *            Question to be displayed
	 * @param presetValue:
	 *            Preset Value (can be an empty string)
	 * @param minValue:
	 *            minimum Value
	 * @param maxValue:
	 *            maximum Value
	 * @return the input Value or null
	 */
	public static Double getDouble(JFrame parentFrame, String question1,
			String presetValue, double minValue, double maxValue) {
		
		boolean hasMax = (maxValue != MAX_DOUBLE);
		boolean hasMin = (minValue != MIN_DOUBLE);
		String question2 = question1.toString();
        
		if (hasMin && hasMax) {
			question2 += "\n(between " + minValue + " and " + maxValue + ")";
		} else {
			if (hasMin) {
				question2 += "\n(>= " + minValue + ")";
			}
			if (hasMax) {
				question2 += "\n(<= " + maxValue + ")";
			}
		}
		
		String valueString = JOptionPane.showInputDialog(parentFrame,
				question2, presetValue);

		if (valueString != null) {
			try {

				Double value = new Double(valueString);
				
				if (value.doubleValue() < minValue
						|| value.doubleValue() > maxValue) {
					
					if (JOptionPane.showOptionDialog(parentFrame, " \""
							+ valueString + "\" is not in Range [" + minValue
							+ "," + maxValue + "].\nClick OK to retype",
							"Warning", JOptionPane.DEFAULT_OPTION,
							JOptionPane.WARNING_MESSAGE, null, OK_CANCEL,
							OK_CANCEL[0]) == 0) {
					
						return getDouble(parentFrame, question1, presetValue,
								minValue, maxValue);
						
					}
				} else {
					return value;
				}
			} catch (NumberFormatException e) {
				
				if (JOptionPane.showOptionDialog(parentFrame, " \""
						+ valueString
						+ "\" is not a double.\nClick OK to retype",
						"Warning", JOptionPane.DEFAULT_OPTION,
						JOptionPane.WARNING_MESSAGE, null, OK_CANCEL, OK_CANCEL[0]) == 0) {
					
					return getDouble(parentFrame, question1, presetValue, minValue, maxValue);
					
				}
			}
		}
		
		return null;
	}

	public static boolean getBoolean(JFrame parentFrame, String question) {
		
		return (JOptionPane.showOptionDialog(parentFrame, question,
				"Please choose", JOptionPane.DEFAULT_OPTION,
				JOptionPane.QUESTION_MESSAGE, null, YES_NO, YES_NO[0]) == 0);
	}

}