package de.ipk_gatersleben.ag_nw.centilib.io;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

/**
 * Save a centrality as a tab separated file.
 * 
 * @author Dirk Koschuetzki
 */
public class TSVFiler<V,E> {

	public void saveVector(String absolutePath,
			VertexScorer<V, Double> ranker,
			Graph<V,E> graph,
			Transformer<V,String> vertexLabelTransformer)
			throws IOException {

		
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(
				absolutePath)));

		int rank = 1;

		pw.print("Position" + "\t" + "VertexLabel" + "\t" + "Value\n");

		for (V v : graph.getVertices()) {
			String label = vertexLabelTransformer.transform(v); 
			if (label == null || label.equalsIgnoreCase(""))
				label = Integer.toString(rank);
			pw.print(String.valueOf(rank));
			pw.print("\t");
			pw.print(String.valueOf(label));
			pw.print("\t");
			// TODO: Format rankScore, perhaps depending on the locale?
			pw.print(ranker.getVertexScore(v));
			pw.print("\n");

			rank++;
		}

		pw.close();
	}
	
	public void saveTable(String absolutePath,
			Graph<V,E> graph,
			List<String> scorerNames,
			Transformer<V,String> vertexLabelTransformer)
			throws IOException{
		
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(absolutePath)));
		readTable(absolutePath,graph,scorerNames,vertexLabelTransformer,pw);
		pw.close();
	}
	
	public void saveMultipleTables(String absolutePath,
			Collection<Graph<V,E> > graphs,
			List<String> scorerNames,
			Transformer<V,String> vertexLabelTransformer)
			throws IOException{
		
		PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(absolutePath)));
		for(Graph<V,E> g : graphs){
			pw.print("graph: "+GraphCachingUtils.getGraphName(g)+"\n\n");
			readTable(absolutePath,g,scorerNames,vertexLabelTransformer,pw);
			pw.print("\n\n");
		}
		pw.close();
	}
	
	/**
	 * <p>
	 * Saves the given centrality values for the nodes in the given graph (numbered consecutively as returned by the graph) using the given PrintWriter.<br />
	 * The following format is used:
	 * </p>
	 * <p>
	 * Position NodeLabel Centrality1 Centrality2 ....Centralitym<br />
	 * 1 label1 value11 value12 ...value1m<br />
	 * 2 label2 value21 value22 ...value2m<br />
	 * .<br />
	 * .<br />
	 * n labeln valuen1 valuen2...valuenm
	 * </p>
	 * 
	 * 
	 * 
	 * @param absolutePath
	 * @param graph
	 * @param scorerNames
	 * @param vertexLabelTransformer
	 * @param pw
	 * @return
	 * @throws IOException
	 */
	protected PrintWriter readTable(String absolutePath,
			Graph<V,E> graph,
			List<String> scorerNames,
			Transformer<V,String> vertexLabelTransformer,
			PrintWriter pw)
			throws IOException{
		
		if(graph == null ||
				scorerNames == null ||
				scorerNames.size() == 0 ||
				vertexLabelTransformer == null ||
				pw == null)
			return pw;
		
		int rank = 1;
		List<String> availableScorerNames = new ArrayList<String>();
		List<VertexScorer<V,Double> > scorers = new ArrayList<VertexScorer<V,Double> >();
		
		// **** get available centralities for the given graph
		VertexScorer<V,Double> tempScorer;
		for(String s : scorerNames){
			tempScorer = GraphCachingUtils.getScorer(graph, s);
			if(tempScorer != null){
				availableScorerNames.add(s);
				scorers.add(tempScorer);
			}
			tempScorer = null;
		}
		
		// **** add column names
		pw.print("Position" + "\t" + "VertexLabel");
		// add the ranker names as column names 
		for(String s : availableScorerNames)
			pw.print("\t"+s);
		pw.print("\n");
		
		Double tempValue = null;
		
		for(V v : graph.getVertices()){
			String label = vertexLabelTransformer.transform(v); 
			if (label == null || label.equalsIgnoreCase(""))
				label = Integer.toString(rank);
			pw.print(rank+"\t"+label);
			
			for(VertexScorer<V,Double> scorer : scorers){
				pw.print("\t");
				tempValue = scorer.getVertexScore(v);
				if(tempValue == null || tempValue.isNaN())
					pw.print("n.a.");
				else
					pw.print(tempValue);
			}
			pw.print("\n");
			rank++;
		}
		
		return pw;
	}
}