package de.ipk_gatersleben.ag_nw.centilib.plugin;

import java.util.Set;

/**
 * This Interface is used to get informations from and interact with unknown data structures that work with graphs.
 * For example, it is used by the CentiLib GUI to access the data structures of the tool it is integrated into.
 * 
 * @author Johannes Graessler
 *
 * @param <G> Type of the graphs
 * @param <V> Type of the nodes
 * @param <E> Type of the edges
 */
public interface CentiLibInteractions<G,V,E> {

	/**
	 * Returns a set of edge attributes for graph that contain numbers or double values respectively
	 * 
	 * @param graph The graph whose edge attributes shall be returned
	 * @return a set of edge attributes that contain numbers
	 */
	public Set<DoubleAttribute<E> > getNumericEdgeAttributes(G graph);


	/**
	 * Returns a set of node attributes for graph that contain numbers or double values respectively
	 * 
	 * @param graph The graph whose node attributes shall be returned
	 * @return a set of node attributes that contain numbers
	 */
	public Set<DoubleAttribute<V> > getNumericNodeAttributes(G graph);

	
	/**
	 * Returns all available graphs
	 * 
	 * @return a set of available graphs
	 */
	public Set<G> getGraphs();

	
	/**
	 * Returns the currently selected graph.
	 * 
	 * @return the currently selected graph 
	 */
	public G getCurrentGraph();
	

	/**
	 * Returns an instance that wraps graph to fit the Interface 'Graph<V,E>'
	 * 
	 * @param graph the graph to be wrapped
	 * @return an instance that wraps graph and implements Graph<V,E>
	 */
	public edu.uci.ics.jung.graph.Graph<V,E> getWrapper(G graph);

	
	/**
	 * Selects all nodes in the currently selected graph view which are contained in the set 'nodes'.
	 * 
	 * @param nodes The nodes to select.
	 */
	public void selectNodesOfCurrentGraph(Set<V> nodes);

	
	/**
	 * Sets 'graph' as current graph and brings the corresponding view into focus (if one exists)
	 * 
	 * @param graph The graph to select.
	 */
	public void selectGraph(G graph);

	
	/**
	 * Adds the attribute centName to node and sets the value 'value'
	 * 
	 * @param node the node to add the attribute to
	 * @param centName the name of the attribute (centrality)
	 * @param value the value of the added attribute
	 * @return true if the attribute was added and the value set, false otherwise
	 */
	public boolean addCentralityAttribute(V node, String centName, Double value);

	
	/**
	 * Deletes the attribute centName from node if it exists
	 *  
	 * @param node the node to delete the attribute from
	 * @param centName the name of the attribute
	 */
	public void deleteCentralityAttribute(V node, String centName);

	
	/**
	 * Creates a subgraph of 'graph' that contains the given nodes and edges. The nodes of
	 * the subgraph should be exact copies of the original ones. This way the visual presentation of the nodes and
	 * their attributes are retained.
	 *  
	 * @param graph original graph
	 * @param nodes set of nodes of the original graph which are to be copied to the subgraph 
	 * @param edges set of edges of the original graph which are to be copied to the subgraph
	 * @param name the name of the subgraph
	 * @return the newly created graph
	 */
	public G createSubGraph(G graph, Set<V> nodes, Set<E> edges, String name);

}
