package de.ipk_gatersleben.ag_nw.centilib.plugin;

import java.awt.Component;
import java.util.Collection;

import org.apache.commons.collections15.Transformer;

/**
 * This class represents a complete instance of CentiLib and it's GUI. 
 * To create it, you need to implement the Interface {@link CentiLibInteractions} which enables
 * CentiLib to access your data structures. Furthermore two Transformers for the graph names and
 * node labels are needed.
 * 
 * Furthermore you have to inform CentiLib if a graph changes. That is, if he was
 * changed or closed or a new graph was opened or focused. Use the appropriate methods to do so.
 * 
 * The GUI (a JTabbedPane) can be accessed with getPanel().
 *  
 * @author Johannes Graessler
 *
 * @param <G> Type of the graphs
 * @param <V> Type of the nodes
 * @param <E> Type of the edges
 */
public class CentiLibPlugin<G,V,E> {

	InteractionHandler<G,V,E> handler;
	
	public CentiLibPlugin(
			CentiLibInteractions<G,V,E> plugin,
			Transformer<G,String> graphNameTransformer,
			Transformer<V, String> nodeLabelTransformer){
		
		
		handler = new InteractionHandler<G,V,E>(plugin,graphNameTransformer,nodeLabelTransformer);
	}
	
	/**
	 * Returns the CentiLib GUI (JTabbedPane).
	 * 
	 * @return the CentiLib GUI
	 */
	public Component getPanel(){
		return handler.getPanel();
	}
	

	/**
	 * Sets the given graph in the panel.
	 * 
	 * @param graph the selected graph
	 */
	public void graphFocused(G graph){
		handler.graphFocused(graph);
	}
	
	/**
	 * Deletes the current graph and all its values within CentiLib.
	 */
	public void graphClosed(){
		handler.graphClosed();
	}
	
	/**
	 * Deletes the given graph and all its values within CentiLib.
	 * 
	 * @param graph The graph to delete from the cache.
	 */
	public void graphClosed(G graph){
		handler.graphClosed(graph);
	}
	
	/**
	 * Marks the current graph as modified. Computed values and existing centrality attributes will be deleted, when the changes are finished.
	 * This is done afterwards because the graph will be already modified.
	 */
	public void preGraphChanged(){
		handler.preGraphChanged();
	}
	
	/**
	 * Marks the given graph as modified. Computed values and existing centrality attributes will be deleted, when the changes are finished.
	 * This is done afterwards because the graph will be already modified.
	 * 
	 * @param graph The graph that will be changed
	 */
	public void preGraphChanged(G graph){
		handler.preGraphChanged(graph);
	}
	
	/**
	 * Deletes all computed values of the current graph within CentiLib and removes existing centrality attributes from the nodes,
	 * because the values get unreliable.
	 */
	public void postGraphChanged(){
		handler.postGraphChanged();
	}
	
	/**
	 * Deletes all computed values of the given graph within CentiLib and removes existing centrality attributes from the nodes,
	 * because the values get unreliable.
	 * 
	 * @param graph the graph that was changed
	 */
	public void postGraphChanged(G graph){
		handler.postGraphChanged(graph);
	}
	
	/**
	 * Marks the appropriate entries for the given nodes in the list of centrality values if they exist (Result panel). 
	 * 
	 * @param nodes the nodes that were selected
	 */
	public void nodesSelected(Collection<V> nodes){
		handler.nodesSelected(nodes);
	}
}
