package de.ipk_gatersleben.ag_nw.centilib.plugin;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections15.BidiMap;
import org.apache.commons.collections15.Transformer;
import org.apache.commons.collections15.bidimap.DualHashBidiMap;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.FileBasedRanker;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.ConnectedComponents;
import de.ipk_gatersleben.ag_nw.centilib.utils.CentralityHandler;
import de.ipk_gatersleben.ag_nw.centilib.utils.GraphCachingUtils;
import edu.uci.ics.jung.algorithms.scoring.VertexScorer;
import edu.uci.ics.jung.graph.Graph;

public class InteractionHandler<G,V,E> implements MouseMotionListener, ApplicationInterface<V,E> {

	private CentiLib<V,E> centilib;
	
	private CentiLibInteractions<G,V,E> plugin;
	
	private Transformer<G,String> graphNameTransformer;

	private BidiMap<G,Graph<V, E> > knownGraphs = new DualHashBidiMap<G,Graph<V,E> >();
	
    /*
     * isSelecting - direction of selecting:
     * If nodes get selected in the panel, they get selected in the graph and vice versa.
     * This member prevents calling this processes concurrently, which would end in a loop.
     */
    private boolean isSelecting = false;
    
	private Map<G,Boolean> isModified = new HashMap<G,Boolean>();
	
	public InteractionHandler(
			//JFrame parent,
			CentiLibInteractions<G,V,E> plugin,
			Transformer<G,String> graphNameTransformer,
			Transformer<V, String> nodeLabelTransformer){
		
		
		centilib = new CentiLib<V,E>(this, nodeLabelTransformer);
		centilib.getPanel().addMouseMotionListener(this);
		this.plugin = plugin;
		this.graphNameTransformer = graphNameTransformer;
		GraphCachingUtils.setUseCache(true);
		//this.centilib.getPanel().addMouseMotionListener(this);
		
	}
	
	public Component getPanel(){
		return centilib.getPanel();
	}
	
	public void graphFocused(G graph){

		if(graph == null){
			centilib.setCurrentGraph(null, "");
			return;
		}
		
		addGraph(graph);
		
		if(isModified.get(graph))
			postGraphChanged(graph);
		else
			centilib.setCurrentGraph(knownGraphs.get(graph), graphNameTransformer.transform(graph));
	}
	
	
	public void preGraphChanged(){
		preGraphChanged(plugin.getCurrentGraph());
	}
	
	public void preGraphChanged(G graph){
		addGraph(graph);
		centilib.deleteGraph(knownGraphs.get(graph));
		isModified.put(graph, true);
	}
	
	public void postGraphChanged(){
		postGraphChanged(plugin.getCurrentGraph());
	}
	
	public void postGraphChanged(G graph){
		addGraph(graph);
		if(graph == null )//|| !isModified.get(graph))
			return;
		
		centilib.deleteGraph(knownGraphs.get(graph));//preGraphChanged(graph);
		deleteAllCentralityAttributes(knownGraphs.get(graph));
		centilib.setCurrentGraph(knownGraphs.get(graph), graphNameTransformer.transform(graph));
		isModified.put(graph, false);
	}
	
	public void graphClosed(){
		graphClosed(plugin.getCurrentGraph());
	}
	
	public void graphClosed(G graph){
		centilib.deleteGraph(knownGraphs.get(graph));
		isModified.remove(graph);
		knownGraphs.remove(graph);
		
	}
	
	public void nodesSelected(Collection<V> nodes){
		if(isSelecting)
			return;
		isSelecting = true;
		centilib.setSelectedVertices(nodes);
		isSelecting = false;
	}
	
	@Override //ApplicationInterface
	public Set<DoubleAttribute<E> > getNumericEdgeAttributes(Graph<V,E> graph){
		if(graph == null || knownGraphs.getKey(graph) == null)
			return null;

		Set<DoubleAttribute<E> > attributes = plugin.getNumericEdgeAttributes(knownGraphs.getKey(graph));
		Set<DoubleAttribute<E> > incomplete = new HashSet<DoubleAttribute<E> >();
		

		if(attributes == null || attributes.size() == 0){
			return null;
		}
		
		for(DoubleAttribute<E> attr : attributes){
			for(E edge : graph.getEdges())
				if(attr.getValue(edge) == null || attr.getValue(edge).isNaN()){
					incomplete.add(attr);
					break;
				}
		}
				
		attributes.removeAll(incomplete);
		return attributes;
	}
	
	@Override //ApplicationInterface
	public void updateGraphList(){
		//if(knownGraphs.size()==0);
		Collection<G> graphs = plugin.getGraphs();
		
		// **** delete closed graphs from centilib
		Collection<G> deletedGraphs = new ArrayList<G>();

		for(G g : knownGraphs.keySet())
			if(!graphs.contains(g)){
				GraphCachingUtils.remove(knownGraphs.get(g));
				deletedGraphs.add(g);
			}
		
		for(G g : deletedGraphs){
			knownGraphs.remove(g);
			isModified.remove(g);
		}
			
		
		// **** add new graphs to centilib
		for(G g : graphs)
			addGraph(g);
		
	}
	
	/**
	 * Tests whether the nodes in graph exist in other graphs, too.
	 *  
	 * @param graph
	 * @return null if all nodes exist only in 'graph', otherwise the first graph one of the nodes exists in, is returned. 
	 */
	protected Graph<V,E> testUniqueNodes(Graph<V,E> graph){
		if(graph == null || knownGraphs.getKey(graph)==null)
			return null;
		
		for(Graph<V,E> g : knownGraphs.values()){
			if(g == graph)
				continue;
			for(V v: graph.getVertices())
				if(g.containsVertex(v))
					return g;
		}
				
		return null;
	}
	
	@Override //ApplicationInterface
	public void addAttributes(Graph<V,E> graph, Collection<String> cents){
		if(graph == null || cents == null || cents.size() == 0)
			return;
		
		Graph<V,E> duplicate = testUniqueNodes(graph);
		if(duplicate != null){
			centilib.showInformationDialog("Error", "One or more nodes to add attributes to, also exist in graph "+
					graphNameTransformer.transform(knownGraphs.getKey(duplicate))+
					".\nPlease close this graph first, since centrality values are only valid for one graph.");
			return;
		}
		
		G destGraph = knownGraphs.getKey(graph);
		String destName = graphNameTransformer.transform(destGraph);
		
		if(destGraph == null || 
				graph == null ||
				graph.getVertexCount() == 0 ||
				cents == null || 
				cents.size()==0 ||
				GraphCachingUtils.getLastScorer(knownGraphs.get(destGraph))==null){
			centilib.showInformationDialog("Hint", "No attributes added for graph '"+destName+"'.");
			return;
		}
		
		Collection<V> centAdded = new HashSet<V>();
		
		for(String cent : cents){
			VertexScorer<V,Double> scorer = GraphCachingUtils.getScorer(graph, cent);
			if(scorer == null) continue;
			centAdded.clear();
			try{
			for(V v : graph.getVertices())
				if(scorer.getVertexScore(v) != null){
					plugin.addCentralityAttribute(v, cent, scorer.getVertexScore(v));
					centAdded.add(v);
				}
				else{
					centilib.showInformationDialog("Error", "Error while adding the " + cent +
							" centrality to node " + centilib.getVertexLabelTransformer().transform(v)+":" +
									"no centrality value found!");
					for(V w : centAdded)
						plugin.deleteCentralityAttribute(w, cent);
					break;
				}
			}
			catch(Exception e){
				centilib.showInformationDialog("Error", "\nAn error occured while adding the " +
						cent + " centrality\n" + "to graph '" +
						graphNameTransformer.transform(knownGraphs.getKey(graph))+"':\n"+e.getMessage());
				for(V w : centAdded)
					plugin.deleteCentralityAttribute(w, cent);
			}
		}

		
	}
	
	@Override //ApplicationInterface
	public void addAttributes(Collection<Graph<V,E> > graphs, Collection<String> cents){
		for(Graph<V,E> g : graphs){
			addAttributes(g,cents);
		}
		
	}
	
	public void deleteAllCentralityAttributes(Graph<V,E> graph){
		Map<DoubleAttribute<V>, LinkedList<Double> > availableCents = getCentralityAttributeMap(graph);
		if(availableCents == null || availableCents.size() == 0)
			return;
		
		for(V node : graph.getVertices())
			for(DoubleAttribute<V> attr : availableCents.keySet())
				plugin.deleteCentralityAttribute(node, attr.getName());
	}
	
	@Override //ApplicationInterface
	public void createGraphsFromComps(ConnectedComponents<V,E> comps){
		String name;
		for(Set<V> comp : comps.getComponents()){
			name = "subgraph("+comp.size()+" nodes) of "+centilib.getCurrentGraphName();
			G newGraph = plugin.createSubGraph(knownGraphs.getKey(centilib.getCurrentGraph()),comp, comps.getEdges(comp),name);
			if(newGraph != null){
				addGraph(newGraph);
				//test if new nodes were created or the nodes exist in both graphs
				//if no new nodes were created, their centrality attributes will be deleted, since they are only reliable for one of the graphs
				if(knownGraphs.get(newGraph).containsVertex(comp.iterator().next()) &&
						getCentralityAttributeMap(centilib.getCurrentGraph()).size()>0)
					centilib.showInformationDialog("Hint", "The subgraph contains the original nodes. " +
							"Therefore existing\ncentrality attributes will be deleted, because they " +
							"are only correct\nfor the graph they were computed for.");
				isModified.put(newGraph, true);
				graphFocused(newGraph);
			}
		}
	}
	
	@Override //ApplicationInterface
	public void selectNodes(Set<V> nodes){
		if(isSelecting)
			return;
		isSelecting = true;
		
		if(nodes != null && nodes.size() != 0)
			plugin.selectNodesOfCurrentGraph(nodes);
		
		isSelecting = false;
	}
	
	@Override //ApplicationInterface
	public void selectGraph(Graph<V,E> graph){
		if(graph == null || isSelecting)
			return;
		
		isSelecting = true;
		plugin.selectGraph(knownGraphs.getKey(graph));
		isSelecting = false;
	}
	
	private void addGraph(G graph){
		if(graph != null && knownGraphs.get(graph) == null){
			Graph<V,E> wrapper = plugin.getWrapper(graph);
			knownGraphs.put(graph, wrapper);
			isModified.put(graph, false);
			addScorerFromAttributes(wrapper);
			GraphCachingUtils.setGraphName(wrapper, graphNameTransformer.transform(graph));
		}
		
	}

	private Map<DoubleAttribute<V>, LinkedList<Double> > getCentralityAttributeMap(Graph<V,E> graph){
		
		/*
		 * This is a little bit complicated since we don't know the exact
		 * attribute name if there was a centrality with a parameter.
		 */
		
		if(graph == null || graph.getVertexCount() == 0) 
			return null;
		
		Map<DoubleAttribute<V>, LinkedList<Double> > availableCents = new HashMap<DoubleAttribute<V>, LinkedList<Double> >();
		Set<DoubleAttribute<V> > incompleteCents = new HashSet<DoubleAttribute<V> >();
		
		Set<DoubleAttribute<V> > attributes = plugin.getNumericNodeAttributes(knownGraphs.getKey(graph));
		
		if(attributes == null || attributes.size() == 0)
			return null;
		
		//find available centrality attributes
		for(String cent : CentralityHandler.getCentList())
			for(DoubleAttribute<V> attr : attributes)
				if(attr.getName().contains(cent))
					availableCents.put(attr,new LinkedList<Double>());
		
		//find centrality attributes that are not complete
		//(that is, not every node has a value for it)
		//at the same time store the valid values in the appropriate linked lists
		for(DoubleAttribute<V> attr : availableCents.keySet()){
			for(V n : graph.getVertices())
			
				
				if(attr.getValue(n) != null && !attr.getValue(n).isNaN())
					availableCents.get(attr).add(attr.getValue(n));
				else{
					incompleteCents.add(attr);
					break;
				}
			
		}

		//remove incomplete centrality attributes
		for(DoubleAttribute<V> attr : incompleteCents)
			availableCents.remove(attr);
		
		return availableCents;
	}
	
	public void addScorerFromAttributes(Graph<V,E> graph){
		
		Map<DoubleAttribute<V>, LinkedList<Double> > availableCents = getCentralityAttributeMap(graph);		
		if(availableCents == null)
			return;
		//add all correctly loaded centralities (each node has a value
		//for a given centrality) to CentiLib
		for(DoubleAttribute<V> attr : availableCents.keySet()){
			LinkedList<Double> values = availableCents.get(attr);
			String centName = attr.getName();
			
			FileBasedRanker<V, E> ranker;				
			ranker = new FileBasedRanker<V, E>(centName,graph, values);
				GraphCachingUtils.addScorer(graph, ranker, centName);
		}

	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		//if any graph changed, opened or closed
		if(isModified.containsValue(true) || plugin.getGraphs().size() != knownGraphs.size()){
				centilib.setCurrentGraph(null, "");
				graphFocused(plugin.getCurrentGraph());
		}
	}
	
	@Override
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


}
