package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.CentiLib;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesCloseness;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesEccentricity;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesRadiality;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesSP;
import de.ipk_gatersleben.ag_nw.centilib.centralities.BrandesStress;
import de.ipk_gatersleben.ag_nw.centilib.centralities.SPBetweenness;
import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.centralities.parameter.CentralityParameter;
import de.ipk_gatersleben.ag_nw.centilib.centralities.parameter.HubbellParameter;
import de.ipk_gatersleben.ag_nw.centilib.centralities.parameter.KatzStatusParameter;
import de.ipk_gatersleben.ag_nw.centilib.centralities.parameter.PageRankParameter;
import de.ipk_gatersleben.ag_nw.centilib.graphstatistics.WienerIndex;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CentroidValuesCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.ClosenessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CurrentFlowBetweennessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.CurrentFlowClosenessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.DegreeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.EccentricityCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.EigenvectorCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.HITSCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.HubbellIndexCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.InOutDegreeCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.KatzStatusIndexCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.PageRankCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.RadialityCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.SPBetweennessCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.centralities.StressCentralityAction;
import de.ipk_gatersleben.ag_nw.centilib.gui.actions.graphstatistics.GraphClusteringCoefficientAction;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DoubleAttribute;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.DistanceStatistics;
import edu.uci.ics.jung.graph.Graph;

/**
 * <p>
 * This class contains only static members and manages all centralities in CentiLib. You can get a list of available centralities with getCentList() or use
 * the static fields for a specific centrality.
 *
 * <p>
 * Use isPossible(graph, centName) to test whether a graph meets the requirements for a given centrality.
 *
 * <p>
 * The method getParameter(centName) returns null
 * if no parameter value is needed for the given centrality. Otherwise it returns an instance of the abstract class {@link CentralityParameter}.
 * If so, you will need to set a parameter value with setValue(value).
 * 
 * <p>
 * To compute a graph or node centrality call getGraphCentralityValue(..) or getScorer(..). See the documentation for details.
 * 
 * <p>
 * To store computed values, activate the cache function with useCache(true). Be sure to remove these values with removeFromCache(graph) if a
 * graph changes. Otherwise you may work with unreliable values.
 * 
 * <p>
 * To compute centralities for a weighted graph, first set the edge weights with GraphCachingUtils.setEdgeWeights(graph, weights) after activating the
 * cache. Afterwards use the CentralityHandler as you would normally do. 
 * 
 * <p>
 * You may also activate a faster computation of shortest-path based centralities by calling useBrandes(true). To start the computations use
 * startBrandes(). For example:
 * </p>
 * 
 * <code>
 * Collection&lt;VertexCentrality&lt;V,E&gt; &gt; cents = new HashSet&lt;VertexCentrality&lt;V, E&gt; &gt;();<br />
 * CentralityHandler.useCache(true);<br />
 * CentralityHandler.useBrandes(true);<br /><br />
 * 
 * if(CentratiyHandler.isPossible(graph,CentralityHandler.STRESS)<br />
 * &nbsp;&nbsp;cents.add( CentralityHandler.getScorer(graph, CentralityHandler.STRESS, null, true));<br /><br />
 * 
 * if(CentralityHandler.isPossible(graph,CentralityHandler.SPBETWEENNESS))<br />
 * &nbsp;&nbsp;cents.add( CentralityHandler.getScorer(graph, CentralityHandler.SPBETWEENNESS, null, true));<br /><br />
 * 
 * CentralityHandler.startBrandes();
 * </code>
 * 
 * @author Johannes Graessler
 *
 */

/*
 * To add a new centrality:
 * 	- new static field with the name
 * 	- in the constructor: add name to centlist and the required conditions
 * 	- add code to method getParam(..) if needed
 * 	- add code to method getScorer(..)
 */

public class CentralityHandler {
	
	public static final String
		CENTROID = 					"Centroid",
		CLOSENESS = 				"Closeness",
		CURRENTFLOWBETWEENNESS = 	"CurrentFlowBetweenness",
		CURRENTFLOWCLOSENESS = 		"CurrentFlowCloseness",
		DEGREE = 					"Degree",
		ECCENTRICITY = 				"Eccentricity",
		EIGENVECTOR = 				"Eigenvector",
		HITSAUTHS = 				"HITSAuths",
		HITSHUBS = 					"HITSHubs",
		HUBBEL = 					"HubbelIndex",
		INDEGREE = 					"InDegree",
		KATZ = 						"KatzStatus",
		OUTDEGREE = 				"OutDegree",
		PAGERANK = 					"PageRank",
		RADIALITY = 				"Radiality",
		SPBETWEENNESS = 			"SPBetweenness",
		STRESS = 					"Stress",
		
		//graph centralities
		AVERAGEDISTANCE = 			"Average Distance",
		CLUSTERINGCOEFFICIENT = 	"Clustering Coefficient",
		DIAMETER =					"Diameter",
		WIENERINDEX = 				"Wiener Index";

	private static List<String> cents = new ArrayList<String>();
	
	private static List<String> graphCents = new ArrayList<String>();
	
	private static String errorMessage = "";
	
	private Map<String, Boolean[]> conditions = new HashMap<String, Boolean[]>();
	
	private static CentralityHandler instance = new CentralityHandler();
		
	private static Boolean useBrandes = false;
	
	private static Map<Graph<?,?>, BrandesSP<?,?> > brandes = new HashMap<Graph<?,?>, BrandesSP<?,?> >();
	
	
	
	public CentralityHandler(){
		
		cents.add(CENTROID);
		cents.add(CLOSENESS);
		//cents.add("Closeness-Vitality");
		cents.add(CURRENTFLOWBETWEENNESS);
		cents.add(CURRENTFLOWCLOSENESS);
		cents.add(DEGREE);
		cents.add(ECCENTRICITY);
		cents.add(EIGENVECTOR);
		cents.add(HITSHUBS);
		cents.add(HITSAUTHS);
		cents.add(HUBBEL);
		cents.add(INDEGREE);
		cents.add(KATZ);
		cents.add(OUTDEGREE);
		cents.add(PAGERANK);
		cents.add(RADIALITY);
		cents.add(SPBETWEENNESS);
		cents.add(STRESS);
		
		graphCents.add(AVERAGEDISTANCE);
		graphCents.add(CLUSTERINGCOEFFICIENT);
		graphCents.add(DIAMETER);
		graphCents.add(WIENERINDEX);
		
		//set the conditions for each centrality
		//Connected, StrongConnected, Undirected, Directed, LoopFree, Simple
		//													con		strcon	undir	dir		lf		simple
		conditions.put(CENTROID,new Boolean[]				{false,	true,	false,	false,	false,	true});
		conditions.put(CLOSENESS, new Boolean[]				{true,	true, 	false, 	false, 	false, 	true});
		conditions.put(CURRENTFLOWBETWEENNESS, new Boolean[]{true, 	false, 	true, 	false, 	false, 	true});
		conditions.put(CURRENTFLOWCLOSENESS, new Boolean[]	{true, 	false, 	true, 	false, 	false, 	true});
		conditions.put(DEGREE, new Boolean[]				{false,	false,	false,	false,	false,	true});
		conditions.put(ECCENTRICITY, new Boolean[]			{true, 	true, 	false, 	false, 	false, 	true});
		conditions.put(EIGENVECTOR, new Boolean[]			{true, 	true, 	false, 	false, 	false, 	true});
		conditions.put(HITSHUBS, new Boolean[]				{true, 	false, 	false, 	false, 	true, 	true});
		conditions.put(HITSAUTHS, new Boolean[]				{true, 	false, 	false, 	false, 	true, 	true});
		conditions.put(HUBBEL, new Boolean[]				{true, 	false, 	false, 	false, 	true, 	true});
		conditions.put(INDEGREE, new Boolean[]				{false,	false,	false,	false,	false,	true});
		conditions.put(KATZ, new Boolean[]					{false, false, 	false, 	false, 	true, 	true});
		conditions.put(OUTDEGREE, new Boolean[]				{false, false, 	false, 	false, 	true, 	true});
		conditions.put(PAGERANK, new Boolean[]				{false, false, 	false, 	false, 	false, 	true});
		conditions.put(RADIALITY, new Boolean[]				{false, false, 	false, 	false, 	false, 	true});
		conditions.put(SPBETWEENNESS, new Boolean[]			{false, false, 	false, 	false, 	false, 	true});
		conditions.put(STRESS, new Boolean[]				{true, 	false/*true*/, 	false, 	false, 	true, 	true});
		
		// graph cents
		conditions.put(AVERAGEDISTANCE, new Boolean[]		{false,	true,	false,	false,	true,	true});
		conditions.put(CLUSTERINGCOEFFICIENT, new Boolean[]	{true,	false,	true,	false,	true,	true});
		conditions.put(DIAMETER, new Boolean[]				{false,	false,	false,	false,	true,	true});
		conditions.put(WIENERINDEX, new Boolean[]			{false,	true,	false,	false,	true,	true});
	}
	
	/**
	 * Returns the list of all known node centralities
	 * 
	 * @return a list of known node centralities
	 */
	public static Collection<String> getCentList(){
		return Collections.unmodifiableCollection(cents);
	}
	
	/**
	 * Returns the list of all known graph centralities.
	 * @return the list of known graph centralities
	 */
	public static Collection<String> getGraphCentList(){
		return Collections.unmodifiableCollection(graphCents);
	}
	
	/**
	 * <p>
	 * Compute the shortest-path based centralities in one run. Use the method getScorer(..) for all needed centralities as you would
	 * normaly do and call startBrandes() afterwards. (see the introduction of this class for an example)<br />
	 * 
	 * <p>
	 * If false is given, pending computations will be started before disabling this feature. Otherwise already created instances which
	 * didn't compute their values yet, would never be able to do so.
	 * </p>
	 * 
	 * @param use sets whether to use this feature or not
	 */
	public static void useBrandes(boolean use){
		useBrandes = use;
		startBrandes();
		brandes.clear();
	}
	
	/**
	 * If useBrandes(true) was called, this method starts the computation of all shortest-path based
	 * centralities which were invoked with getScorer(..).
	 */
	public static void startBrandes(){

		for(BrandesSP<?,?>  inst: brandes.values()){
			if(inst != null)
				inst.start();
		}
		brandes.clear();
	}
	
	/**
	 * Activate or deactivate the cache. If the cache is activated, all values like 
	 * graph properties and centralities will be computed only once. Therefore, make sure 
	 * to remove a graphs values if it changes (removeFromCache(graph)).
	 * 
	 * @param use sets whether to use the cache or not
	 */
	public static void useCache(boolean use){
		GraphCachingUtils.setUseCache(use);
	}
	
	/**
	 * Removes the given graph and all its values from the cache.
	 * 
	 * @param graph the graph to remove 
	 */
	public static void removeFromCache(Graph<?,?> graph){
		GraphCachingUtils.remove(graph);
	}
	
	private static <V,E> void addToBrandes(Graph<V,E> graph, BrandesSPListener<V,E> listener){
		if(brandes.get(graph) == null){
			Transformer<E, ? extends Number> edgeWeights = GraphCachingUtils.getEdgeWeightTransformer(graph);
			if(edgeWeights == null)
				brandes.put(graph, new BrandesSP<V,E>(graph));
			else
				brandes.put(graph, new BrandesSP<V, E>(graph, edgeWeights));
		}
		((BrandesSP<V,E>)brandes.get(graph)).addListener(listener);
	}
	
	/**
	 * Tests whether a graph meets the requirements for the given centrality,
	 * i.e. is connected, simple or loop free.<br />
	 * If a centrality is not possible for a given graph, you can get the errors
	 * with getErrorMessage(). It only returns a string, which combines all errors that occured
	 * and was introduced to inform the user.
	 *  
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph the graph to test
	 * @param centName the centrality to test the graph for
	 * 
	 * @return true if the given centrality can be computed for the graph, false otherwise
	 */
	public static <V,E> boolean isPossible(Graph<V,E> graph, String centName){
		if(instance == null) return false;
		
		if(graph == null || centName==null || centName.equals("") || instance.conditions.get(centName) == null)
			return false;
		
		Boolean[] con = instance.conditions.get(centName);
		if(con == null || con.length != 6) return false;
		
		if(GraphPreconditionChecks.checkPreconditions(graph,con[0], con[1],con[2],con[3],con[4],con[5]))
			return true;
			
		errorMessage = GraphPreconditionChecks.getErrorMessage();
		return false;
	}
	
	/**
	 * <p>
	 * Use the method getParameter(graph, centName) instead.
	 * 
	 * <p>
	 * If the given centrality needs a parameter value, this method will return an instance
	 * of the abstract class CentralityParameter. The CentiLib instance is used to ask the user
	 * for a value (only needed, if the CentiLib GUI is used).
	 * If you want to ask him later use false for 'setValue' and call the method askUserForValue() later.
	 *  
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param centilib used to ask the user for a value
	 * @param graph the graph the given centrality shall be computed for
	 * @param centName the centrality to get the parameter for, if nedded
	 * @param setValue whether to ask the user for a value now or not
	 * 
	 * @return an instance of {@link CentralityParameter} if the given centrality needs a parameter value, null otherwise
	 */
	public static <V,E> CentralityParameter getParameter(CentiLib<V,E> centilib, Graph<V,E> graph, String centName, boolean setValue){
		CentralityParameter
		param = null;
		
		if(graph == null || centName == null || centName.equals(""))
			return null;
		
		// **** check for all known centralities, that needs a parameter
		if(centName.equals(KATZ)) param = new KatzStatusParameter<V, E>(centilib, graph); else
		if(centName.equals(HUBBEL)) param = new HubbellParameter<V, E>(centilib,graph); else
		if(centName.equals(PAGERANK)) param = new PageRankParameter(centilib,graph);
		
		if(param != null && setValue)
			param.askUserForValue();
		
		return param;
	}
	
	/**
	 * If the given centrality needs a parameter value, this method returns an instance of the abstract class
	 * {@link CentralityParameter}. You have to set a value with setValue(value). You can get the range
	 * with getMinValue() and getMaxValue();
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph the graph the given centrality shall be computed for
	 * @param centName the centrality to get the parameter for
	 * @return an instance of CentralityParameter if needed for the given centrality, null otherwise
	 */
	public static <V,E> CentralityParameter getParameter(Graph<V,E> graph, String centName){
		return getParameter(null,graph,centName,false);
	}
	
	/**
	 * Returns a String with errors that occured in one of this classes methods.
	 * This method was introduced to inform the user about errors. It's not possible
	 * to determine single errors.<br />
	 * After this method is called, the errors are deleted.
	 * 
	 * @return a String with errors that occured since the last call of this method
	 */
	public static String getErrorMessage(){
		String temp = errorMessage;
		errorMessage = "";
		return temp;
	}
	
	/**
	 * Returns a list with the names of already computed centralities for the given graph.
	 * That is, all centralities for a given Graph, that exist in GraphCachingUtils.<br />
	 * To use this function, the cache has to be activated with useCache(true) before computing
	 * centralities.
	 * 
	 *  @param graph the graph to get already computed centralities for
	 *	@return the names of centralities which are already computed for the given graph
	 */
	public static <V,E> Collection<String> getAvailableCentNames(Graph<V,E> graph){
		return GraphCachingUtils.getAvailableScorerNames(graph);
	}
	
	/**
	 * Computes the graph centralities.
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph the graph to compute the centrality
	 * @param centName the centrality to compute
	 * @param compute if true: compute the centrality if it does not exist in the cache
	 * @return the value for the given graph centrality; returns null if not possible or 'compute' is set to false and the value is not available in the cache 
	 */
	public static <V,E> Double getGraphCentralityValue(Graph<V,E> graph, String centName, boolean compute){
		if(
				graph == null ||
				centName == null ||
				centName.equals("") || 
				!isPossible(graph,centName))
			
			return null;
		
		if(GraphCachingUtils.getGraphCentrality(graph, centName) != null)
			return GraphCachingUtils.getGraphCentrality(graph, centName);
		
		if(!compute)
			return null;
		
		Double result = null;
		Distance<V> distance = GraphCachingUtils.getDistance(graph);
		
		if(centName.equals(AVERAGEDISTANCE)){
			Double wienerIndex = getGraphCentralityValue(graph,WIENERINDEX,true);
			if(wienerIndex == null){
				result = null;
			}
			else{
				int n = graph.getVertexCount();
				result = (wienerIndex / (n * (n - 1)));
			}
		}else
			
		if(centName.equals(CLUSTERINGCOEFFICIENT)) result = GraphClusteringCoefficientAction.calculateClusteringCoefficient(graph);else
		if(centName.equals(DIAMETER)) result = DistanceStatistics.diameter(graph, distance,true); else
		if(centName.equals(WIENERINDEX)) result = new WienerIndex<V,E>(graph,distance).getWienerIndex();
		
		if(result != null)
			GraphCachingUtils.addGraphCentrality(graph, centName, result);
		
		return result;
	}
	
	/**
	 * <p>
	 * Returns an instance of {@link VertexCentrality} for the given centrality. If the centrality needs a parameter
	 * use getParameter(graph, centName). The parameter 'createScorer' determines whether to
	 * compute the centrality, if it isn't already available in the cache.
	 * 
	 * <p>
	 * If useBrandes(true) was called, shortest-path based centralities will be computed immediatly. 
	 * Instead getScorer(.., true) will return an instance, which does not contain any values yet.
	 * You also have to start the appropriate computations by calling startBrandes(). The following example shows how
	 * to compute all centralities, which do not need a parameter, for a given graph.
	 * </p>
	 * 
	 * <code>
	 * CentralityHandler.useCache(true);<br />
	 * 
	 * CentralityHandler.useBrandes(true);<br /><br />
	 * 
	 * Collection&lt;VertexCentrality&lt;V,E&gt; &gt; cents = new HashSet&lt;VertexCentrality&lt;V,E&gt; &gt;();<br />
	 * VertexCentrality&lt;V,E&gt; cent;<br /><br />
	 * 
	 * for(String centName : CentralityHandler.getCentList())<br />
	 * &nbsp;&nbsp;if(CentralityHandler.isPossible(graph,centName)){<br />
	 * &nbsp;&nbsp;&nbsp;&nbsp;cent = CentralityHandler.getScorer(graph,centName,null,true);<br />
	 * &nbsp;&nbsp;&nbsp;&nbsp;if(cent != null)<br />
	 * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;cents.add(cent);<br />
	 * &nbsp;&nbsp;}<br />
	 * CentralityHandler.startBrandes();
	 * </code>
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to compute the given centrality for. 
	 * @param centName The name of the requested centrality.
	 * @param param The parameter for the given centrality, if needed, null otherwise.
	 * @param createScorer Determines whether to compute the centrality if it is not available in the cache or not.
	 * @return an instance of the given centrality if possible, null otherwise
	 */
	@SuppressWarnings("unchecked")
	public static <V,E> VertexCentrality<V,E> getScorer(Graph<V,E> graph, String centName, CentralityParameter param, boolean createScorer){
		if(graph == null || centName == null || centName.equals(""))
			return null;
		
		VertexCentrality<V,E> scorer = GraphCachingUtils.getScorer(graph, centName);

		if(scorer != null)
			return scorer;
		if(!createScorer)
			return null;
//		if(!isPossible(graph,centName))
//			return null;
		
		Distance<V> distance = GraphCachingUtils.getDistance(graph);
		Transformer<E, ? extends Number> edgeWeightTransformer = GraphCachingUtils.getEdgeWeightTransformer(graph);
		Map<E, Number> edgeWeights = GraphCachingUtils.getEdgeWeights(graph);
		
		if(param != null)
			if(param.getValue() != null){
		
				if(centName.equals(PAGERANK)) 
					scorer =  PageRankCentralityAction.calculatePageRank(graph, param.getValue(), 50,edgeWeights);
				if(centName.equals(KATZ))
					scorer =  KatzStatusIndexCentralityAction.calculateKatzStatusIndex(graph, param.getValue(),edgeWeights);
				if(centName.equals(HUBBEL))
					scorer =  HubbellIndexCentralityAction.calculateHubbellIndex(graph, param.getValue(),true);
			}
			else{
				errorMessage = "Parameter for " + centName + " was not set correctly and returned null";
				return null;
			}
		
		if(centName.equals(CENTROID)) scorer =  CentroidValuesCentralityAction.calculateCentroidValues(graph,distance);else

		if(centName.equals(CLOSENESS))
			if(useBrandes)
				scorer = new BrandesCloseness<V,E>(graph);
			else scorer =  ClosenessCentralityAction.calculateCloseness(graph, edgeWeightTransformer);else
		if(centName.equals(CURRENTFLOWBETWEENNESS)) scorer =  CurrentFlowBetweennessCentralityAction.calculateCurrentFlowBetweenness(graph);else
		
		if(centName.equals(CURRENTFLOWCLOSENESS)) scorer =  CurrentFlowClosenessCentralityAction.calculateCurrentFlowCloseness(graph);else
		
		if(centName.equals(DEGREE)) scorer = DegreeCentralityAction.calculateDegree(graph);else
		
		if(centName.equals(ECCENTRICITY))
			if(useBrandes)
				scorer = new BrandesEccentricity<V, E>(graph);
			else scorer =  EccentricityCentralityAction.calculateEccentricity(graph, edgeWeightTransformer);else
				
		if(centName.equals(EIGENVECTOR)) scorer =  EigenvectorCentralityAction.calculateEigenvector(graph, edgeWeights);else
		
		if(centName.equals(HITSAUTHS)) scorer =  HITSCentralityAction.calculateHITS(graph, true, 50);else
		
		if(centName.equals(HITSHUBS)) scorer =  HITSCentralityAction.calculateHITS(graph, false, 50);else
			
		if(centName.equals(INDEGREE)) scorer =  InOutDegreeCentralityAction.calculateInOutDegree(graph, true);else
			
		if(centName.equals(OUTDEGREE)) scorer =  InOutDegreeCentralityAction.calculateInOutDegree(graph, false);else
			
		if(centName.equals(RADIALITY))
			if(useBrandes)
				scorer = new BrandesRadiality<V, E>(graph);
			else scorer =  RadialityCentralityAction.calculateRadiality(graph,edgeWeightTransformer);else
			
		if(centName.equals(SPBETWEENNESS))
			if(useBrandes)
				scorer = new SPBetweenness<V, E>(graph);
			else scorer =  SPBetweennessCentralityAction.calculateSPBetweenness(graph,edgeWeightTransformer);else
			
		if(centName.equals(STRESS))
			if(useBrandes)
				scorer = new BrandesStress<V, E>(graph);
			else scorer =  StressCentralityAction.calculateStress(graph, edgeWeightTransformer);
		
		if(scorer != null){
			if(param != null){
				DecimalFormat df = new DecimalFormat("#.##");
				String paramString= df.format(param.getValue());
				paramString = paramString.replace(',', '.');
				
				//cut after two decimal places
//				if(paramString.indexOf('.') < paramString.length()-3)
	//				paramString = paramString.substring(0,paramString.indexOf(".")+3);
				centName += paramString;
			}
			DoubleAttribute<E> attr = GraphCachingUtils.getLastEdgeAttribute(graph);
			if(attr != null)
				centName += "_weight_"+attr.getName();
			
			if(useBrandes && scorer instanceof BrandesSPListener<?,?>)
				addToBrandes(graph, (BrandesSPListener<V,E>)scorer);
			
			GraphCachingUtils.addScorer(graph, scorer, centName);
		}
		else{
			instance.errorMessage = "The requested Scorer '"+centName+"' could not be found or the parameter is missing.\n";
		}
		return scorer;
	}
}
