package de.ipk_gatersleben.ag_nw.centilib.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections15.BidiMap;
import org.apache.commons.collections15.Transformer;

import de.ipk_gatersleben.ag_nw.centilib.centralities.VertexCentrality;
import de.ipk_gatersleben.ag_nw.centilib.plugin.DoubleAttribute;
import edu.uci.ics.jung.algorithms.shortestpath.DijkstraDistance;
import edu.uci.ics.jung.algorithms.shortestpath.Distance;
import edu.uci.ics.jung.algorithms.shortestpath.ShortestPath;
import edu.uci.ics.jung.algorithms.shortestpath.UnweightedShortestPath;
import edu.uci.ics.jung.graph.Graph;

/**
 * <p>
 * All values like centrality values, graph properties, edge weights or connected components can be stored and accessed via this class.
 * As default, the cache is not activated to avoid unreliable values when a graph changes. If you want to store values activate
 * the cache via setUseCache(true) and make sure to remove a graph and its values if a node or an edge was added or removed because already
 * computed values get unreliable.<br />
 * If the cache is activated, every time a method is called for an unknown graph an instance of {@link Distance} is created. It computes
 * and stores the distances between the nodes of a graph. If edge weights are set, weighted shortest paths will be computed. Make use
 * of this instance via getDistance(graph), to compute a shortest path only if needed and only once.
 * 
 * <p>
 * If a value is not available yet, it will
 * be computed (and stored if the chache is activated) automatically. This is not true for centralities. Use {@link CentralityHandler}
 * to compute centralities. 
 * 
 * @author Johannes Graessler
 *
 */
//@SuppressWarnings("unchecked")
public class GraphCachingUtils<V,E> {
	
	private static GraphCachingUtils instance = new GraphCachingUtils();
	
	private Boolean useCache = false;
	
	private Map<Graph<V,E>,Boolean> isConnected  = new HashMap<Graph<V,E>,Boolean>();				//<Graph, Boolean>

	private Map<Graph<V,E>,Boolean> isStrongConnected  = new HashMap<Graph<V,E>,Boolean>();		

	private Map<Graph<V,E>,Boolean> containsSelfLoops= new HashMap<Graph<V,E>,Boolean>();

	private Map<Graph<V,E>,Boolean> containsParallelEdges = new HashMap<Graph<V,E>,Boolean>();

	private Map<Graph<V,E>,Boolean> isDirected = new HashMap<Graph<V,E>,Boolean>();

	private Map<Graph<V,E>,Boolean> isUndirected = new HashMap<Graph<V,E>,Boolean>();
	
	private Map<Graph<V,E>,Distance<V>> distance = new HashMap<Graph<V,E>,Distance<V>>();
	
	// **** graphCents
	private Map<Graph<V,E>,Map<String,Double> > graphCentralities = new HashMap<Graph<V,E>,Map<String,Double> > (); 
	
	private Map<Graph<V,E>,Map<VertexCentrality<V,E>, BidiMap<V,Integer> >> sortedResultList 
		= new HashMap<Graph<V,E>,Map<VertexCentrality<V,E>, BidiMap<V,Integer> >> ();		//<Graph, Map<VertexScorer<V>, BidiMap<V,Integer> > >
	
	private Map<Graph<V,E>,Set<Set<V> > > connectedComponents = new HashMap<Graph<V,E>,Set<Set<V> > > ();		//<Graph, Set< Set<V> > >		-> a list of connected components for a given graph
	
	private Map<Graph<V,E>,Collection<Collection<V>>> strongConnectedComponents = new HashMap<Graph<V,E>,Collection<Collection<V>>>();	//<Graph, Collection<Collection<V > > >	-> a list of strongly connected components
	
	private Map<Collection<V>, Collection<E>> edgesOfComponent = new HashMap<Collection<V>,Collection<E>>();		//<Collection<V>, Collection<E> >

	private Map<Graph<V,E>, String> knownGraphs = new HashMap<Graph<V,E>, String>();				//<Graph<V,E>, String> -> Graph and his name
	/*
	 * Maps a Graph to a Pair(Rankername,Ranker(VertexScorer)), if there is a ranker with the given name that
	 * already worked with the graph.
	 */
	private Map<Graph<V,E>,Map<String,VertexCentrality<V,E> > > rankerListForGraph = new HashMap<Graph<V,E>,Map<String,VertexCentrality<V,E> > >();
	
	private Map<Graph<V,E>, Map<E, Double> > edgeWeights = new HashMap<Graph<V,E>, Map<E, Double> >();
	
	/*
	 * Maps a graph to the last ranker (VertexScorer), that worked with this graph 
	 */
	private Map<Graph<V,E>,MyPair<V>> lastRankerForGraph = new HashMap<Graph<V,E>, MyPair<V> >();
	
	private Map<Graph<V,E>, DoubleAttribute<E> > lastEdgeAttributeForGraph = new HashMap<Graph<V,E>, DoubleAttribute<E> >();
	
	
	/**
	 * Sets whether to store computed values or not.
	 * 
	 * @param useCache True if values should be stored.
	 */
	public static void setUseCache(boolean useCache){
		instance.useCache = useCache;
		if(!useCache)
			removeAllGraphsFromCache();
	}
	
	/**
	 * Stores the name of a graph.
	 * 
	 * @param <V> Type of nodes.
	 * @param <E> Type of edges.
	 * @param graph The graph to store the name for.
	 * @param name The name of the given graph.
	 */
	public static <V,E> void setGraphName(Graph<V,E> graph, String name){
		
		if(!validateGraph(graph) || name == null) return;
		
		instance.knownGraphs.put(graph, name);
	}
	
	/**
	 * Returns the name of the given graph, if it exists.
	 * 
	 * @param <V> Type of nodes.
	 * @param <E> Type of edges.
	 * @param graph The graph to return the name for.
	 * @return The name of the given graph. Null if it does not exist.
	 */
	public static <V,E> String getGraphName(Graph<V,E> graph){
		if(!validateGraph(graph)) return "";
		return (String) instance.knownGraphs.get(graph);
	}
	
	/**
	 * Stores the given weights for the graph 'graph'. If no weights are given, existing ones are deleted.
	 * Returns false, if useCache is set to false or graph is null;
	 * 
	 * @param <V> Type of the vertices
	 * @param <E> Type of the edges
	 * @param graph The graph for which the edge weights will be stored
	 * @param weights A Map to resolve the weights for all edges
	 * @return True if useCache is set to true, false otherwise
	 */
	public static <V,E> boolean setEdgeWeights(Graph<V,E> graph, Map<E,Number> weights){
		if(!validateGraph(graph) || !instance.useCache)
			return false;
		
		// if no weights are given, delete existing ones
		if(weights == null){
			removeEdgeWeights(graph);
			return true;
		}
		else{
			
			Map<E,Double> doubleWeights = new HashMap<E,Double>();
			
			for(E edge : weights.keySet())
				doubleWeights.put(edge, weights.get(edge).doubleValue());
			
			instance.edgeWeights.put(graph,doubleWeights);

			Transformer<E,? extends Number> transformer = getEdgeWeightTransformer(graph);

			//((Map<String,VertexCentrality<V,E> > )instance.rankerListForGraph.get(graph)).clear();
			//instance.lastRankerForGraph.remove(graph);
			instance.distance.put(graph, new DijkstraDistance<V, E>(graph, transformer,true));
		}
		return true;
	}
	
	/**
	 * Sets the current edge attribute for the given graph. This only stores the attribute for reference. No edge weights
	 * will be generated. If null is given, the last attribute will be deleted for the given graph. 
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to store the attribute for.
	 * @param attribute The attribute to store.
	 */
	public static <V,E> void setLastEdgeAttribute(Graph<V,E> graph, DoubleAttribute<E> attribute){
		if(!validateGraph(graph))
			return;
		if(attribute == null){
			instance.lastEdgeAttributeForGraph.remove(graph);
			return;
		}
		
		instance.lastEdgeAttributeForGraph.put(graph, attribute);
	}
	
	/**
	 * Removes existing edge weights for the given graph. The last edge attribute will be deleted, too.
	 * Furthermore the weighted distances will be replaced by unweighted ones.
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to remove the edge weights for.
	 */
	public static <V,E> void removeEdgeWeights(Graph<V,E> graph){

		if(instance.edgeWeights.get(graph) == null)
				return;
			
		instance.edgeWeights.remove(graph);
		instance.lastEdgeAttributeForGraph.remove(graph);
		//instance.lastRankerForGraph.remove(graph);
		//((Map<String,VertexCentrality<V,E> > )instance.rankerListForGraph.get(graph)).clear();
		instance.distance.put(graph, new UnweightedShortestPath<V, E>(graph));
		
	}
	
	/**
	 * Returns the edge weights for the given graph, if they exist. Otherwise it returns null.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to get edge weights for.
	 * @return the edge weights for the given graph if they exist, null otherwise.
	 */
	public static <V,E> Map<E,Number> getEdgeWeights(Graph<V,E> graph){
		if(!validateGraph(graph))
			return null;
		
		return (Map<E,Number>) instance.edgeWeights.get(graph);
	}

	/**
	 * Returns a transformer for edge weights if they were set. Call transformer.transform(edge) to use it.
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to get the transformer for.
	 * @return a transformer for edge weights or null
	 */
	public static <V,E> Transformer<E,Double> getEdgeWeightTransformer(Graph<V,E> graph){
		if(!validateGraph(graph))
			return null;
		
		if(getEdgeWeights(graph) == null)
			return null;
		
		final Map<E,? extends Number> edgeWeights = (Map<E, ? extends Number>) getEdgeWeights(graph);
		Transformer<E,Double> transformer = new Transformer<E,Double>(){
				
				public Double transform(E edge){
					return edgeWeights.get(edge).doubleValue();
				}
			};
			
		return transformer; 
	}
	
	/**
	 * Returns the last edge attribute for the given graph if it exists. Otherwise null is returned.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to get the attribute for.
	 * @return the last attribute for the given graph or null
	 */
	public static <V,E> DoubleAttribute<E> getLastEdgeAttribute(Graph<V,E> graph){
		return (DoubleAttribute<E>) instance.lastEdgeAttributeForGraph.get(graph);
	}

	/**
	 * Returns the list of known graphs.
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @return the list of known graphs
	 */
	public static <V,E> Collection<Graph<V,E> > getGraphs(){//Graph<V,E> graph){
		return Collections.unmodifiableCollection(instance.knownGraphs.keySet());
	}
	
	/**
	 * Returns the requested centrality value for the given graph if it exists. Otherwise null is returned. 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph a centrality should be returned for
	 * @param centName The name of the centrality to get for the given graph
	 * @return the graph centrality value for the given graph or null
	 */
	public static <V,E> Double getGraphCentrality(Graph<V,E> graph, String centName){
		if(!validateGraph(graph))
			return null;
		return (Double) ((Map<String,Double>) instance.graphCentralities.get(graph)).get(centName);
	}
	
	/**
	 * Returns an instance of the interface {@link Distance} for the given graph. It computes and stores the
	 * lengths of shortest paths for a single graph. If edge weights were set, they will be taken into account.
	 * 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to get the distances for.
	 * @return an instance for weighted or unweighted lengths of shortest paths for the given graph
	 */
	public static <V,E> Distance<V> getDistance(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;
		return (Distance<V>) instance.distance.get(graph);
	}
	
	/**
	 * Returns an instance of the interface {@link ShortestPath} for the given graph. In contrast to
	 * the interface Distance<V> you get the exact shortest paths instead of the length.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to shortest paths for.
	 * @return an instance for the shortest paths of the given graph.
	 */
	public static <V,E> ShortestPath<V,E> getShortestPath(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;
		return (ShortestPath<V,E>) instance.distance.get(graph);
	}
	
	/**
	 * Determines whether the given graph is connected or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph
	 * @return true if graph is connected, false otherwise
	 */
	public static <V,E> boolean isConnected(Graph<V,E> graph){
		if(!validateGraph(graph)) return false;
		if(instance.isConnected.get(graph) == null)
			instance.isConnected.put(graph, GraphCachingUtils.getConnectedComponents(graph).size() == 1);
		
		return (Boolean) instance.isConnected.get(graph);
	}
	
	/**
	 * Determines whether graph is strongly connected or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to check.
	 * @return true if graph is strongly connected, false otherwise.
	 */
	public static <V,E> boolean isStrongConnected(Graph<V,E> graph){
		
		if(!validateGraph(graph)) return false;
		if(instance.isStrongConnected.get(graph)== null){
			Set<Set<V> > comps = GraphCachingUtils.getStrongConnectedComponents(graph);
			instance.isStrongConnected.put(graph, comps.size() == 1);
			//instance.isStrongConnected.put(graph, GraphProperties.isStrongConnected(graph,(Distance<V>) instance.distance.get(graph)));
		}
		
		return (Boolean) instance.isStrongConnected.get(graph);
	}
	
	/**
	 * Determines whether graph is directed or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to check.
	 * @return true if graph is directed, false otherwise. 
	 */
	public static <V,E> boolean isDirected(Graph<V,E> graph){
		
		if(!validateGraph(graph)) return false;
		if(instance.isDirected.get(graph)== null)
			instance.isDirected.put(graph, GraphProperties.isDirected(graph));
		
		return (Boolean) instance.isDirected.get(graph);
	}
	
	/**
	 * Determines whether graph is undirected or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to check.
	 * @return true if graph is undirected, false otherwise.
	 */
	public static <V,E> boolean isUndirected(Graph<V,E> graph){
		
		if(!validateGraph(graph)) return false;
		if(instance.isUndirected.get(graph)== null)
			instance.isUndirected.put(graph, GraphProperties.isUndirected(graph));
		
		return (Boolean) instance.isUndirected.get(graph);
	}

	/**
	 * Determines whether graph contains self loops or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to find self loops in.
	 * @return true if graph contains self loops, false otherwise.
	 */
	public static <V,E> boolean containsSelfLoops(Graph<V,E> graph){
		
		if(!validateGraph(graph)) return false;
		if(instance.containsSelfLoops.get(graph) == null)
			instance.containsSelfLoops.put(graph, GraphProperties.containsSelfLoops(graph));
		
		return (Boolean) instance.containsSelfLoops.get(graph);
	}
	
	/**
	 * Determines whether graph contains parallel edges or not.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to find parallel edeges in.
	 * @return true if graph contains parallel edges.
	 */
	public static <V,E> boolean containsParallelEdges(Graph<V,E> graph){
		
		if(!validateGraph(graph)) return false;
		if(instance.containsParallelEdges.get(graph) == null)
			instance.containsParallelEdges.put(graph, GraphProperties.containsParallelEdges(graph));
		
		return (Boolean) instance.containsParallelEdges.get(graph);
	}
	
	/**
	 * Adds the given graph centrality value for graph to the cache. 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to store the centrality value for
	 * @param centName The name of the graph centrality
	 * @param value The value of the graph centrality
	 */
	public static <V,E> void addGraphCentrality(Graph<V,E> graph, String centName, Double value){
		if(!validateGraph(graph)) return;
		
		if(getGraphCentrality(graph, centName) == null)
			((Map<String,Double>)instance.graphCentralities.get(graph)).put(centName, value);
		
	}
	
	/**
	 * Adds the given node centrality for graph to the cache.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to store the centrality for.
	 * @param scorer The instance that contains the centrality values.
	 * @param name The name of the centrality
	 */
	public static <V,E> void addScorer(Graph<V,E> graph, VertexCentrality<V,E> scorer, String name){
		if(!instance.useCache || !validateGraph(graph)) return;
		if(scorer == null)
			return;
		
		if(getScorer(graph, name)==null){
			((Map<String,VertexCentrality<V,E> >)instance.rankerListForGraph.get(graph)).put(name, scorer);
			/*Collection<MyPair<V>> list = (Collection<MyPair<V>>) instance.rankerListForGraph.get(graph);
			list.add(new MyPair<V>(scorer,name));*/
		}
		
		instance.lastRankerForGraph.put(graph, new MyPair<V>(scorer,name));
			
	}
	
	/**
	 * Returns the currently selected centrality instance for graph. That is, the last centrality added.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to return the lastly added centrality for.
	 * @return the last centrality that was added for graph or null 
	 */
	public static <V,E> VertexCentrality<V,E> getLastScorer(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;
		MyPair<V> pair = (MyPair<V>)instance.lastRankerForGraph.get(graph);
		if(pair != null)
			//this cast is not safe, because MyPair gets a graph of type Graph<V,?>,
			//but in general the type should be the same
			return (VertexCentrality<V,E>)pair.getScorer();
		return null;
	}
	
	/**
	 * Returns the name of the last scorer that was added to the cache.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to get the name of the lastly added centrality for.
	 * @return the name of the last centrality that was added for graph or an empty string.
	 */
	public static <V,E> String getLastScorerName(Graph<V,E> graph){
		if(!validateGraph(graph)) return "";
		MyPair<V> pair = (MyPair<V>)instance.lastRankerForGraph.get(graph);
		if(pair != null)
			return pair.getName();
		return "";
	}
	
	/**
	 * Returns the centrality with name scorerName for the given graph.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to return the centrality for.
	 * @param scorerName The name of the centrality 
	 * @return a centrality instance or null
	 */
	public static <V,E> VertexCentrality<V,E> getScorer(Graph<V,E> graph, String scorerName){
		if(!validateGraph(graph)) return null;
		/*Collection<MyPair<V>> list = (Collection<MyPair<V>>) instance.rankerListForGraph.get(graph);
		if(list == null)
			return null;
		for(MyPair<V> pair : list){
			if(pair.getName().equals(scorerName))
				return pair.getScorer();
		}
		return null;*/
		return ((Map<String,VertexCentrality<V,E> >)instance.rankerListForGraph.get(graph)).get(scorerName);
	}
	
	/**
	 * Returns the list of names of already computed centralities for the given graph.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph
	 * @return the list of names of available centralities. 
	 */
	public static <V,E> Collection<String> getAvailableScorerNames(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;
		
		return Collections.unmodifiableCollection(((Map<String,VertexCentrality<V,E> >)instance.rankerListForGraph.get(graph)).keySet());
	}

	/**
	 * This method is for CentiLib internal use only (result panel).
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph !document me!
	 * @param ranker !document me!
	 * @return !document me!
	 */
	public static <V,E> BidiMap<V,Integer> getResultList(Graph<V,E> graph, VertexCentrality<V,E> ranker){
		if(!validateGraph(graph)) return null;
		return (BidiMap<V,Integer>) ((HashMap<VertexCentrality<V,E>, BidiMap<V,Integer> >)instance.sortedResultList.get(graph)).get(ranker);
	}
	
	/**
	 * This method is for CentiLib internal use only (result panel).
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph !document me!
	 * @param ranker !document me!
	 * @param list !document me!
	 */
	public static <V,E> void setResultList(Graph<V,E> graph, VertexCentrality<V,E> ranker, BidiMap<V,Integer> list){
		if(getResultList(graph, ranker) != null)
			return;
		
		if(!validateGraph(graph)) return;
		((HashMap<VertexCentrality<V,E>, BidiMap<V,Integer> >) instance.sortedResultList.get(graph)).put(ranker,list);
	}

	/**
	 * Returns a set of connected components for the given graph.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to return the connected components for.
	 * @return a set of connected components.
	 */
	public static <V,E> Set<Set<V> > getConnectedComponents(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;

		if(instance.connectedComponents.get(graph) == null)
			instance.connectedComponents.put(graph, GraphUtils.getConnectedComponents(graph));	
		
		return (Set<Set<V> >)instance.connectedComponents.get(graph);
	}
	
	/**
	 * Returns a set of strongly connected components for the given graph.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to return the strongly connected components for.
	 * @return a set of strongly connected components or null.
	 */
	public static <V,E> Set<Set<V> > getStrongConnectedComponents(Graph<V,E> graph){
		if(!validateGraph(graph)) return null;
		
		if(instance.strongConnectedComponents.get(graph) == null)
			instance.strongConnectedComponents.put(graph, GraphUtils.getStrongConnectedComponents(graph));
		
		return (Set<Set<V > >)instance.strongConnectedComponents.get(graph);
	}
	
	/**
	 * Returns the edges between the given vertices.
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph the vertices and edges belong to.
	 * @param vertices The nodes to get the edges for.
	 * @return the edges that exist betwenn the given nodes in the graph or null.
	 */
	public static <V,E> Set<E> getEdgesOfComponent(Graph<V,E> graph, Set<V> vertices){
		if(!validateGraph(graph)) return null;
		if(instance.edgesOfComponent.get(vertices) == null)
			instance.edgesOfComponent.put(vertices, GraphUtils.getEdgesBetweenVertices(graph, vertices));
		
		return (Set<E>) instance.edgesOfComponent.get(vertices);
	}
	
	/**
	 * This class is used to store a centrality and it's name.
	 * 
	 * @author Johannes Graessler
	 *
	 * @param <V> Type of nodes
	 */
	private static class MyPair<V>{
		private String name;
		private VertexCentrality<V,?> scorer;
		
		public MyPair(VertexCentrality<V,?> scorer, String name){
			this.name = name;
			this.scorer = scorer;
		}
		
		public String getName(){return name;}
		public VertexCentrality<V,?> getScorer(){return scorer;}
	}
	
	/**
	 * Removes the given graph and all it's values from the cache. 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to remove from the cache.
	 */
	public static <V,E> void remove(Graph<V,E> graph){
		
		if(graph == null || instance.knownGraphs.get(graph)==null)
			return;
		
		//System.out.println("\tgraph ist bekannt:\n\tAnzahl Knoten: "+graph.getVertexCount());
		instance.sortedResultList.remove(graph);
		instance.graphCentralities.remove(graph);
		instance.distance.remove(graph);
		instance.isConnected.remove(graph);
		instance.isStrongConnected.remove(graph);
		instance.isUndirected.remove(graph);
		instance.isDirected.remove(graph);
		instance.containsParallelEdges.remove(graph);
		instance.containsSelfLoops.remove(graph);
		instance.rankerListForGraph.remove(graph);
		instance.lastRankerForGraph.remove(graph);
		instance.knownGraphs.remove(graph);
		instance.edgeWeights.remove(graph);
		instance.lastEdgeAttributeForGraph.remove(graph);
				
		if (instance.connectedComponents.get(graph) != null) {
			for (Set<V> comp : (Set<Set<V>>) instance.connectedComponents.get(graph))
				instance.edgesOfComponent.remove(comp);
			instance.connectedComponents.remove(graph);

		}
		
		if (instance.strongConnectedComponents.get(graph) != null) {
			for (Set<V> comp : (Set<Set<V>>) instance.strongConnectedComponents.get(graph))
				instance.edgesOfComponent.remove(comp);
			instance.strongConnectedComponents.remove(graph);
		}
	}
	
	/**
	 * Removes all graphs from the cache. That is, cleans it.
	 */
	public static void removeAllGraphsFromCache(){
		Collection<Graph<?,?> > graphs = new HashSet<Graph<?,?> >(instance.knownGraphs.size());
		graphs.addAll(instance.knownGraphs.keySet());
		
		for(Graph<?,?> g : graphs)
			remove(g);		
	}
	
	/**
	 * If the given graph is unknown and the cache is used, an instance of {@link Distance} and some hash maps will be generated. 
	 * @param <V> Type of nodes
	 * @param <E> Type of edges
	 * @param graph The graph to validate
	 * @return True if the is not null, false otherwise
	 */
	private static<V,E> boolean validateGraph(Graph<V,E> graph){
		if(graph == null) return false;
		if(!instance.useCache){
			removeAllGraphsFromCache();
		}
		
		if(instance.knownGraphs.get(graph) == null){
			instance.knownGraphs.put(graph,"");
			
			UnweightedShortestPath<V, E> usp = new UnweightedShortestPath<V, E>(graph);
			instance.distance.put(graph,usp);
			instance.graphCentralities.put(graph, new HashMap<String,Double>());
			//instance.rankerListForGraph.put(graph, new HashMap<Graph<V,E>, Map<String,VertexCentrality<V,E> > >());
			instance.rankerListForGraph.put(graph, new HashMap<String,VertexCentrality<V,E> >());
			instance.sortedResultList.put(graph, new HashMap<VertexCentrality<V,E>, BidiMap<V,Integer> >());
		}
		return true;
	}
	
}
